# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'Ui_DTDialog.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from DTPySide.DTWidget.DTTitleBar import DTTitleBar


class Ui_DTDialog(object):
    def setupUi(self, DTDialog):
        if not DTDialog.objectName():
            DTDialog.setObjectName(u"DTDialog")
        DTDialog.resize(433, 314)
        DTDialog.setStyleSheet(u"")
        self.verticalLayout = QVBoxLayout(DTDialog)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.TitleBar = DTTitleBar(DTDialog)
        self.TitleBar.setObjectName(u"TitleBar")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.TitleBar.sizePolicy().hasHeightForWidth())
        self.TitleBar.setSizePolicy(sizePolicy)

        self.verticalLayout.addWidget(self.TitleBar)

        self.centralWidget = QHBoxLayout()
        self.centralWidget.setObjectName(u"centralWidget")
        self.centralWidget.setContentsMargins(41, 10, 32, 15)

        self.verticalLayout.addLayout(self.centralWidget)

        self.buttonBoxLayout = QHBoxLayout()
        self.buttonBoxLayout.setSpacing(0)
        self.buttonBoxLayout.setObjectName(u"buttonBoxLayout")
        self.buttonBoxLayout.setContentsMargins(-1, -1, 32, 25)
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.buttonBoxLayout.addItem(self.horizontalSpacer)

        self.buttonBox = QDialogButtonBox(DTDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.buttonBox.sizePolicy().hasHeightForWidth())
        self.buttonBox.setSizePolicy(sizePolicy1)
        self.buttonBox.setLayoutDirection(Qt.RightToLeft)
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(False)

        self.buttonBoxLayout.addWidget(self.buttonBox)


        self.verticalLayout.addLayout(self.buttonBoxLayout)


        self.retranslateUi(DTDialog)
        self.buttonBox.accepted.connect(DTDialog.accept)
        self.buttonBox.rejected.connect(DTDialog.reject)

        QMetaObject.connectSlotsByName(DTDialog)
    # setupUi

    def retranslateUi(self, DTDialog):
        DTDialog.setWindowTitle(QCoreApplication.translate("DTDialog", u"DTDialog", None))
    # retranslateUi




from __future__ import annotations
from DTPySide import *

class DTPlainTextEdit(QPlainTextEdit):
	"""
	A TextEdit editor that sends editingFinished events 
	when the text was changed and focus is lost.
	"""

	editingFinished = Signal()
	receivedFocus = Signal()
	# edited=Signal()
	# returnPressed=Signal()

	def __init__(self, parent):
		super().__init__(parent)
		# self.setTabChangesFocus( True )

	def focusInEvent(self, event):
		super().focusInEvent(event)
		self.receivedFocus.emit()

	def focusOutEvent(self, event):
		super().focusOutEvent(event)
		self.editingFinished.emit()
	
	# def keyPressEvent(self,event):
	# 	super().keyPressEvent( event )
	# 	if event.key()==Qt.Key_Return:
	# 		self.returnPressed.emit()
	# 	self.edited.emit()
		


from __future__ import annotations
from DTPySide import *

class MarkdownViewer(QTextBrowser):

	# def eventFilter(self, watched: QObject, event:QEvent) -> bool:
	
	# 	if event.type()==QEvent.ShortcutOverride:
	# 		self.shortcut_queue.append(event.key())
	# 		self.shortcut_queue.pop(0)
	# 		if self.shortcut_queue==[16777249, 67]:
	# 			print("Copied")
	# 			# self.copyMarkdown()
	# 		return True #好像不起作用……
	# 	return False

	def __init__(self, parent):
		super().__init__(parent=parent)
		# self.installEventFilter(self)
		# self.shortcut_queue=[0, 0]
	
	def contextMenuEvent(self, e: QContextMenuEvent):
		menu=self.createStandardContextMenu()

		new_CopyAction=QAction("Copy Markdown")
		new_CopyAction.triggered.connect(self.copyMarkdown)
		menu.insertAction(menu.actions()[1], new_CopyAction)
		
		menu.exec_(e.globalPos())
		del menu
	
	def copyMarkdown(self):
		clip=QGuiApplication.clipboard()
		
		# 只能给我html……
		html=self.textCursor().selection().toHtml()
		
		# 找个做苦力的来转换哈哈哈哈
		temp=QTextEdit()
		temp.setHtml(html)
		text=temp.toMarkdown()
		
		clip.setText(text)


from __future__ import annotations
from DTPySide import *

# Setting 
from DTPySide.DTModule.Ui_DTSetting import Ui_DTSetting
class DTSetting(Ui_DTSetting,QWidget):
	def __init__(self, Headquarter:DTSession.DTMainSession, app:DTAPP):
		super().__init__(Headquarter)
		self.setupUi(self)
		
		self.Headquarter=Headquarter
		self.app=app

		self.initializeWindow()
		self.initializeSignal()

	def initializeWindow(self):
		
		#加入第一页的menu button
		BasicInfoPageButton=DTWidget.DTSettingButton(IconFromCurrentTheme("settings.svg"))
		self.addPageButton(BasicInfoPageButton,0)
		
		
		if self.app.isLoginEnable():
			self.lineEdit_password.setText(self.app.password())
		else:
			self.lineEdit_password.hide()
			self.label_password.hide()
		
		if self.app.DataList()!=[]:
			self.lineEdit_data_dir.setText(self.app.DataDir())
			
			if self.app.isBackupEnable():
				dst=self.app.BackupDst()
				if dst==None:
					self.lineEdit_backup.setText("")
				else:
					self.lineEdit_backup.setText(dst)
			else:
				self.label_backup.hide()
				self.lineEdit_backup.hide()
				self.pushButton_backup.hide()
		
		else:
			self.lineEdit_data_dir.hide()
			self.label_data_dir.hide()
			self.pushButton_data_dir.hide()
			self.label_backup.hide()
			self.lineEdit_backup.hide()
			self.pushButton_backup.hide()

		# font
		self.pushButton_font.setText(self.app.Font().key().split(",")[0])
		self.pushButton_font.setStyleSheet("font-family:%s; min-height:26px; max-height:26px;"%self.app.Font().key().split(",")[0])

		# scale
		self.spinBox_scale.setValue(self.app.Scale())
		self.comboBox_window_effect.setCurrentIndex(["Normal","Aero","Acrylic"].index(self.app.WindowEffect()))
		
		# theme
		self.comboBox_theme.addItems(self.app.ThemeList)
		self.comboBox_theme.setCurrentIndex(self.app.ThemeList.index(self.app.Theme()))

		# color
		self.setStyleSheet("""
		QSlider{
			padding-left:10px;
			padding-right:10px;
			height: 36px;
		}
		QSlider::groove:horizontal {
			height: 24px;
			border: none;
			border-radius: 6px;
		}
		QSlider::handle {
			height: 28px;
		}
		QSlider::add-page{
			background:transparent;
		}
		QSlider::sub-page{
			background: transparent;
		}
		""")
		# hue
		self.slider_Hue.setValue(int(self.app.Hue()*self.slider_Hue.maximum()))
		self.slider_Hue.setStyleSheet("""
		QSlider::groove{
			background:qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(255, 0, 0, 255), stop:0.166 rgba(255, 255, 0, 255), stop:0.333 rgba(0, 255, 0, 255), stop:0.5 rgba(0, 255, 255, 255), stop:0.666 rgba(0, 0, 255, 255), stop:0.833 rgba(255, 0, 255, 255), stop:1 rgba(255, 0, 0, 255));
		}
		""")
		self.pushButton_hue_reset.setStyleSheet("QPushButton{ min-height:16px; max-height:16px; min-width:16px; max-width:16px; icon-size:12px; }")
		self.pushButton_hue_reset.setIcon(IconFromCurrentTheme("corner-down-left.svg"))
		# saturation
		self.slider_saturation.setValue(int(self.app.Saturation()*self.slider_saturation.maximum()))
		self.updateSaturationSlider()
		self.pushButton_saturation_reset.setStyleSheet("QPushButton{ min-height:16px; max-height:16px; min-width:16px; max-width:16px; icon-size:12px; }")
		self.pushButton_saturation_reset.setIcon(IconFromCurrentTheme("corner-down-left.svg"))
		# luminance
		self.slider_luminance.setValue(int(self.app.Luminance()*self.slider_luminance.maximum()))
		self.slider_luminance.setStyleSheet("""
		QSlider::groove{
			background:qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(0, 0, 0, 255), stop:1 rgba(255, 255, 255, 255));
		}
		""")
		self.pushButton_luminance_reset.setStyleSheet("QPushButton{ min-height:16px; max-height:16px; min-width:16px; max-width:16px; icon-size:12px; }")
		self.pushButton_luminance_reset.setIcon(IconFromCurrentTheme("corner-down-left.svg"))

		self.slider_contrast.setValue(int(self.app.Contrast()*self.slider_contrast.maximum()))
		self.slider_contrast.setStyleSheet("""
		QSlider::groove{
			background:qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(255, 255, 255, 255), stop:1 rgba(0, 0, 0, 255));
		}
		""")
		self.pushButton_contrast_reset.setStyleSheet("QPushButton{ min-height:16px; max-height:16px; min-width:16px; max-width:16px; icon-size:12px; }")
		self.pushButton_contrast_reset.setIcon(IconFromCurrentTheme("corner-down-left.svg"))
		
		self.updateColorPreview()

		if self.app.Reverse():
			self.checkBox_reverse.setCheckState(Qt.Checked)

		if self.app.hasTanslation():
			self.comboBox_language.addItems(self.app.translation_module.Language_Dict.keys())
			self.comboBox_country.addItems(self.app.translation_module.Country_Dict.keys())
			self.comboBox_language.setCurrentText(self.app.Language())
			self.comboBox_country.setCurrentText(self.app.Country())
		else:
			self.comboBox_language.hide()
			self.label_language.hide()
			self.comboBox_country.hide()
			self.label_country.hide()

		self.setMinimumSize(800,550)
		
	def initializeSignal(self):
		self.pushButton_font.clicked.connect(self.FontSetting)
	
		if self.app.isLoginEnable():
			self.lineEdit_password.editingFinished.connect(self.PasswordSetting)
		
		self.pushButton_data_dir.clicked.connect(self.DataDirSetting)
		
		if self.app.isBackupEnable():
			self.pushButton_backup.clicked.connect(self.BackupSetting)
		
		self.pushButton_scale.clicked.connect(self.ScaleSetting)
		self.comboBox_window_effect.currentIndexChanged.connect(self.WindowEffectSetting)
		self.comboBox_theme.currentIndexChanged.connect(self.ThemeSetting)
		
		self.pushButton_color.clicked.connect(self.ColorSetting)

		def slotColorPreview():
			self.updateColorPreview()
			self.updateSaturationSlider()
		
		self.slider_Hue.sliderMoved.connect(slotColorPreview)
		self.slider_saturation.sliderMoved.connect(slotColorPreview)
		self.slider_luminance.sliderMoved.connect(slotColorPreview)

		self.pushButton_hue_reset.clicked.connect(self.HueReset)
		self.pushButton_saturation_reset.clicked.connect(self.SaturationReset)
		self.pushButton_luminance_reset.clicked.connect(self.LuminanceReset)
		self.pushButton_contrast_reset.clicked.connect(self.ContrastReset)
		
		self.checkBox_reverse.stateChanged.connect(self.ReverseSetting)
	
		if self.app.hasTanslation():
			self.comboBox_language.currentIndexChanged.connect(self.LanguageSetting)
			self.comboBox_country.currentIndexChanged.connect(self.CountrySetting)

	def PasswordSetting(self):
		
		try:
			self.app.setPassword(self.lineEdit_password.text())
			self.Headquarter.saveAllEncryptData()
			DTFrame.DTMessageBox(self,"Information","Password reseted to \"%s\" successfully!"%self.app.password(),DTIcon.Information())
		except:
			DTFrame.DTMessageBox(self,"Warning","Error occur during password reseting!",DTIcon.Error())

	def DataDirSetting(self):
		dlg=QFileDialog(self)
		data_dir=dlg.getExistingDirectory()
		if data_dir:

			warning_list=[]
			for file in self.app.DataList():
				if file in os.listdir(data_dir):
					warning_list.append(file)
			
			flag=True
			if warning_list!=[]:
				if DTFrame.DTConfirmBox(
					self,
					"Warning",
					"There are already files with the same names in the new Data Dir: \n\n%s\n\nDo you want to overwrite them?"%data_dir,
					icon=DTIcon.Warning(),
					detail="\n".join(warning_list)
				).exec_():
					flag=True
				else:
					flag=False
			
			if flag==True:
				self.app.setDataDir(data_dir)
				self.lineEdit_data_dir.setText(self.app.DataDir())
				DTFrame.DTMessageBox(self,"Information","Data Dir changed to\n\n \"%s\" \n\nsuccessfully!"%self.app.DataDir(),DTIcon.Information())

	def BackupSetting(self):
		dlg=QFileDialog(self)
		backup_dst=dlg.getExistingDirectory()
		if backup_dst:
			self.app.setBackupDst(backup_dst)
			self.lineEdit_backup.setText(self.app.BackupDst())
			DTFrame.DTMessageBox(self,"Information","Backup Dst changed to\n\n \"%s\" \n\nsuccessfully!"%self.app.BackupDst(),DTIcon.Information())
	
	def FontSetting(self):
		ok, font = QFontDialog.getFont(self.app.Font(), self)
		if ok:
			self.app.setFont(font)
			# DTFrame.DTMessageBox(self,"Information","Font changed to \"%s\" successfully!\n\nApp will be restart for better experience."%self.app.Font().key().split(",")[0],DTIcon.Information())
			self.app.restart()	
	
	def ScaleSetting(self):
		self.app.setScale(round(self.spinBox_scale.value(),1))
		# DTFrame.DTMessageBox(self,"Information","Scale changed to \"%s\" successfully!\n\nApp will be restart for better experience."%self.app.Scale(),DTIcon.Information())
		self.app.restart()

	def WindowEffectSetting(self):
		self.app.setWindowEffect(self.comboBox_window_effect.currentText())
		# DTFrame.DTMessageBox(self,"Information","Window Effet changed to \"%s\" successfully!\n\nApp will be restart for better experience."%self.app.WindowEffect(),DTIcon.Information())
		self.app.restart()

	def ThemeSetting(self):
		self.app.setTheme(self.comboBox_theme.currentText())
		self.app.restart()
	
	def updateColorPreview(self):
		if self.app.Hue()==-1 and self.slider_Hue.value()==0:
			self.label_color_preview.setStyleSheet("background:black")
			return
		c=colour.Color()
		c.set_hue(float(self.slider_Hue.value()/self.slider_Hue.maximum()))
		c.set_saturation(float(self.slider_saturation.value()/self.slider_saturation.maximum()))
		c.set_luminance(float(self.slider_luminance.value()/self.slider_luminance.maximum()))
		color=c.get_web()
		self.label_color_preview.setStyleSheet("background:%s"%color)

	def updateSaturationSlider(self):
		if self.app.Hue()==-1 and self.slider_Hue.value()==0:
			self.slider_saturation.setStyleSheet("""
			QSlider::groove{
				background:qlineargradient(spread:pad, x1:0, y1:0.5, x2:1, y2:0.5, stop:0 rgba(255, 255, 255, 255), stop:1 rgba(0, 0, 0, 255));
			}
			""")
			return
		c=colour.Color()
		c.set_hue(float(self.slider_Hue.value()/self.slider_Hue.maximum()))	
		c.set_luminance(0.5)
		c.set_saturation(1.0)
		r,g,b=list(map(lambda x:int(x*255),c.get_rgb()))
		self.slider_saturation.setStyleSheet("""
		QSlider::groove{
			background:qlineargradient(spread:pad, x1:0, y1:0.5, x2:1, y2:0.5, stop:0 rgba(255, 255, 255, 255), stop:1 rgba(%s, %s, %s, 255));
		}
		"""%(r,g,b))

	def ColorSetting(self):
		if self.app.Hue()==-1 and self.slider_Hue.value()==0:
			pass
		else:
			self.app.setHue(float(self.slider_Hue.value()/self.slider_Hue.maximum()))
		self.app.setSaturation(float(self.slider_saturation.value()/self.slider_saturation.maximum()))
		self.app.setLuminance(float(self.slider_luminance.value()/self.slider_luminance.maximum()))
		self.app.setContrast(float(self.slider_contrast.value()/self.slider_contrast.maximum()))
		self.app.initializeWindowStyle(refresh=True)

	def HueReset(self):
		self.app.setHue(-1)
		self.slider_Hue.setValue(0)
		self.app.initializeWindowStyle(refresh=True)
		self.updateColorPreview()
		self.updateSaturationSlider()
	
	def SaturationReset(self):
		self.app.setSaturation(0.5)
		self.slider_saturation.setValue(int(self.slider_saturation.maximum()/2))
		self.app.initializeWindowStyle(refresh=True)
	
	def LuminanceReset(self):
		self.app.setLuminance(0.5)
		self.slider_luminance.setValue(int(self.slider_luminance.maximum()/2))
		self.app.initializeWindowStyle(refresh=True)
	
	def ContrastReset(self):
		self.app.setContrast(0.5)
		self.slider_contrast.setValue(int(self.slider_contrast.maximum()/2))
		self.app.initializeWindowStyle(refresh=True)

	def ReverseSetting(self):
		dlg=DTFrame.DTConfirmBox(self,"Warning","You want to change text and icon from dark to light (or from light to dark)?\n\nApp will restart after the change.")
		if dlg.exec_():
			if self.checkBox_reverse.checkState()==Qt.Checked:
				self.app.setReverse(True)
			else:
				self.app.setReverse(False)
			self.app.restart()
	
	def LanguageSetting(self):
		self.app.setLanguage(self.comboBox_language.currentText())
		# DTFrame.DTMessageBox(self,"Information","Language changed to \"%s\" successfully!\n\nApp will be restart for better experience."%self.app.Language(),DTIcon.Information())
		self.app.restart()

	def CountrySetting(self):
		self.app.setCountry(self.comboBox_country.currentText())
		# DTFrame.DTMessageBox(self,"Information","Country changed to \"%s\" successfully!\n\nApp will be restart for better experience."%self.app.Country(),DTIcon.Information())
		self.app.restart()
	
	def appendStackPage(self,page):
		index=self.stackedWidget.addWidget(page)
		return index

	def addPageButton(self,button,index):
		"传入一个按钮，自动加入到ButtonMenu列表的队尾，并链接好跳转到该位置的stackwidget page的信号"
		
		button.clicked.connect(lambda:self.stackedWidget.setCurrentIndex(index))
		self.verticalLayout_buttons.insertWidget(self.verticalLayout_buttons.count()-1,button)
		
		###################################
		# 以前还费劲用listwidget装载pushbutton，明明用layout这么方便，就是却不会用……
		# item=QListWidgetItem()
		# 
		# font_size=self.font().pointSize()*4
		# item.setSizeHint(QSize(font_size,font_size))
		# 
		# # 就是这个鬼头东西
		# # 如果不设置一下
		# # 右键列表空白处时就会出现一个标识QListWidgetItem的小长方形
		# # 它和button的大小不一致，就暴露了马脚
		# # 找了半个小时才找到……
		# 
		# self.listWidget_buttons.addItem(item)
		# self.listWidget_buttons.setItemWidget(item,button)
		###################################


from __future__ import annotations
from DTPySide import *

# Setting
class DTSettingSession(DTFrame.DTDialog):
	def __init__(self, Headquarter, app):
		super().__init__(Headquarter,"Setting")
		
		# 不要按钮了，实时保存设置
		self.buttonBox.hide()
		self.centralWidget.setContentsMargins(QMargins(9,10,32,10))
		self.buttonBoxLayout.setContentsMargins(QMargins(0,0,32,0))
		
		self.__SettingModule=DTModule.DTSetting(Headquarter, app)
		self.setCentralWidget(self.__SettingModule)

		self.resize(self.minimumWidth(),self.minimumHeight())
		self.adjustSize()
		MoveToCenterOfScreen(self)

		self.setStyleSheet("QStackedWidget QPushButton{ min-height:36px }")
	
	def addButtonAndPage(self,button,qwidget):
		"传入一个button和stackwidget page中的QWidget，button将自动加入到ButtonMenu列表的队尾，并链接好跳转到该stackwidget page的信号"
		index=self.__SettingModule.appendStackPage(qwidget)
		self.__SettingModule.addPageButton(button,index)


from __future__ import annotations
from DTPySide import *

# APP
class DTAPP(QApplication):
	"""DTAPP注释


	Args:
		QApplication ([type]): [description]
	"""
	
	def __init__(self, args):

		self.__UserSetting=QSettings("./UserSetting.ini",QSettings.IniFormat)
		
		# 添加缩放大小的环境变量
		os.environ["QT_SCALE_FACTOR"] = str(self.Scale())

		# 设置图片缩放质量
		self.setAttribute(Qt.AA_UseOpenGLES,True)
		# self.setAttribute(Qt.AA_EnableHighDpiScaling,True)
		self.setAttribute(Qt.AA_UseHighDpiPixmaps,True)
		
		# 设置字体缩放质量
		# QLibraryInfo will load qt.conf from one of the following locations:
		#	1. :/qt/etc/qt.conf using the resource system
		#	2. on macOS, in the Resource directory inside the application bundle, for example assistant.app/Contents/Resources/qt.conf
		#	3. in the directory containing the application executable, i.e. QCoreApplication::applicationDirPath() + QDir::separator() + "qt.conf"
		# 
		# print(self.applicationDirPath()+QDir.separator()) to see where the qt.conf should be placed
		# 新建一个qt.conf写下下面的设置 would cause the Windows platform plugin to use the FreeType font engine.
		# [Platforms]
		# WindowsArguments = fontengine=freetype
		# 这样字体也就不会有锯齿啦~
		#
		# （用Python解释器跑的时候，self.applicationDirPath()==D:/Program/Python，也就是python.exe的根目录
		# 如果这时候把qt.conf放到D:/Program/Python，会影响对相应组件的加载与打包（即使设置了Prefix等路径也不行）
		# 
		# 所以在解释的时候就不放qt.conf，而是在打包后，把qt.conf放在main.exe的目录下
		# （如果用了WebEngine的话，还要复制一份resource、translation和QtWebEngineProcess.exe到根目录）

		# 继承stylesheet
		self.setAttribute(Qt.AA_UseStyleSheetPropagationInWidgetStyles,True)
		
		# BossKey隐藏主窗口后不退出QApp
		self.setQuitOnLastWindowClosed(False)
		self.setQuitOnClickX(True)
		
		super().__init__(args)
		
		self.setStyle("Fusion")
		self.ThemeList=["Dracula","Dracula2","Brown","Green","Cyan","White"]
		self.initializeWindowStyle()
		
		self.TrayIcon=QSystemTrayIcon(self)

		self.setWindowIcon(DTIcon.HoloIcon1())
		self.setApplicationName("DT's Project")
		self.setApplicationVersion("0.0.0.0")
		self.setAuthor("鍵山狐")
		self.setOrganizationName("Dongli Teahouse")
		self.setOrganizationDomain("dongliteahouse.wordpress.com")
		self.setContact("Holence08@gmail.com")
		
		self.setDataList([])
		self.setLoginEnable(False)
		self.setBackupEnable(False)
	
	def initializeTrayIcon(self):
		"生成TrayIcon"
		
		self.TrayIcon.setIcon(self.windowIcon())
		self.TrayIcon.activated.connect(self.__mainsession.windowResurrection)
		self.TrayIcon.setContextMenu(self.__mainsession._MainMenu)
		
		self.TrayIcon.show()

	def initializeWindowStyle(self, refresh=False):
		"""生成color_list，并设置MainwWindow的Window Effect和Theme
		"""

		def changeColor(color_list,new_hue:float,new_saturation:float,new_luminance:float,contrast:float,reverse:bool):
			hue_offset=new_hue-colour.Color(color_list[1]).get_hue()
			sat_offset=new_saturation-0.5
			lum_offset=new_luminance-0.5

			if reverse:
				color=colour.Color(color_list[-1])
				r,g,b=color.get_rgb()
				r=1-r
				g=1-g
				b=1-b
				color=colour.Color()
				color.set_rgb((r,g,b))
				color_list[-1]=color.get_web()
			
			contrast=contrast-0.5
			std_lum=max(min(colour.Color(color_list[1]).get_luminance()+lum_offset,1),0)

			for i in range(len(color_list)):
				color=colour.Color(color_list[i])
				if new_hue!=-1:
					color.set_hue((color.get_hue()+hue_offset)%360)
				
				if i>=len(color_list)-3:
					new_saturation=max(min(color.get_saturation()+sat_offset*0.5,1),0)
					new_luminance=max(min(color.get_luminance()+lum_offset*0.5,1),0)
					new_luminance=max(min(new_luminance+(new_luminance-std_lum)*contrast*0.5,1),0)
				else:
					new_saturation=max(min(color.get_saturation()+sat_offset,1),0)
					new_luminance=max(min(color.get_luminance()+lum_offset,1),0)
					new_luminance=max(min(new_luminance+(new_luminance-std_lum)*contrast,1),0)
				
				color.set_saturation(new_saturation)
				color.set_luminance(new_luminance)
				
				color_list[i]=color.get_web()
			
			return color_list

		# DEEPDARK="#191A21" # Border和GroupBox、TitleBarFrame的Background
		# BACKGROUND="#21222C" # background
		# SOFTDARK="#282A36" # LineEdit、QPushButton的背景
		# DIM="#404257" # disable的文字、Itemview的item的背景
		# PRESSED = "#A67DB4" # Button Clicked 
		# FOCUSED="#8C6BBB" # Button Hover、text selection
		# TEXT="#E0E0E0" # 文字
		# ICONCOLOR="white" # 部分icon的颜色（暂未适配ui文件中指派的icon）
		
		white_or_black=["white","black"]
		theme=self.Theme()
		hue=self.Hue()
		saturation=self.Saturation()
		luminance=self.Luminance()
		contrast=self.Contrast()
		reverse=self.Reverse()

		if theme=="Dracula":
			self.color_list=["#191A21","#21222C","#282A36","#404257", "#A67DB4","#8C6BBB","#E0E0E0"]
			QIcon.setThemeName(white_or_black[reverse])
		elif theme=="Dracula2":
			self.color_list=["#202329","#282C34","#313341","#404257","#D7AAE6","#BD93F9","#EBEBEB"]
			QIcon.setThemeName(white_or_black[reverse])
		elif theme=="Brown":
			self.color_list=["#202020","#2A2A2A","#353535","#5c5c5c","#7AB6F3","#2A82DA","#FFFFFF"]
			QIcon.setThemeName(white_or_black[reverse])
		elif theme=="Green":
			self.color_list=["#17241F","#294137","#3F6151","#5D796C","#C6CA8F","#A5AD79","#EEF1E0"]
			QIcon.setThemeName(white_or_black[reverse])
		elif theme=="Cyan":
			self.color_list=["#1C242D","#303F53","#3F5670","#5b789e","#B0C8D2","#8BACBC","#EEF4ED"]
			QIcon.setThemeName(white_or_black[reverse])
		elif theme=="White":
			self.color_list=["#aaaaaa","#ffffff","#dddddd","#9B9B9B","#eeeeee","#dae3ea","#333333"]
			QIcon.setThemeName(white_or_black[not reverse])
		
		if hue!=-1 or saturation!=0.5 or luminance!=0.5 or contrast!=0.5:
			self.color_list=changeColor(self.color_list,hue,saturation,luminance,contrast,reverse)

		self.setStyleSheet(DTStyleSheet(self.color_list, self.WindowEffect(), self.Font()))

		if refresh==True:
			self.__mainsession.refresh()

	def hasTanslation(self):
		return hasattr(self,"translation")

	def loadTranslation(self, translation_module=DTTranslation):
		"""请在构造MainSession之前调用这个函数，因为MainSession中的文字会用translate来请求translation
		
		设置translation的module（请复制DTTranslation文件夹到自己的项目中，先翻译自己项目里的文本，之后再在lrelease的时候与合并qm文件）。
		
		如果不调用这个函数，App默认为无translation，设置菜单中也不会有语言和地区的选项

		Args:
			translation_module (module, optional): Defaults to DTTranslation.
		"""		
		
		self.translation_module=translation_module

		if self.Language() not in translation_module.Language_Dict:
			self.setLanguage("English")
		if self.Country() not in translation_module.Country_Dict:
			self.setCountry("World")
		
		language=self.Language()
		country=self.Country()
		
		# set QLocale
		QLocale.setDefault(QLocale(translation_module.Language_Dict[language][0],translation_module.Country_Dict[country]))

		# load .qm file
		self.translation=QTranslator()
		self.translation.load(translation_module.Language_Dict[language][1])
		self.installTranslator(self.translation)
	
	def Font(self) -> QFont:
		if self.UserSetting().value("BasicInfo/Font")==None:
			font=QFont()
			font.setFamily("Meiryo UI")
			font.setPointSize(18)
			self.setFont(font)
		return self.UserSetting().value("BasicInfo/Font")
	
	def setFont(self, font:QFont):
		self.UserSetting().setValue("BasicInfo/Font",font)

	def Scale(self):
		if self.UserSetting().value("BasicInfo/Scale")==None or float(self.UserSetting().value("BasicInfo/Scale"))<1:
			self.setScale(1.0)
		return float(self.UserSetting().value("BasicInfo/Scale"))
	
	def setScale(self, scale:float):
		self.UserSetting().setValue("BasicInfo/Scale",str(scale))
	
	def WindowEffect(self):
		if self.UserSetting().value("BasicInfo/WindowEffect") not in ["Normal","Aero","Acrylic"]:
			self.setWindowEffect("Normal")
		return self.UserSetting().value("BasicInfo/WindowEffect")
	
	def setWindowEffect(self, WindowEffect:str):
		self.UserSetting().setValue("BasicInfo/WindowEffect",WindowEffect)
	
	def Theme(self):
		if self.UserSetting().value("BasicInfo/Theme") not in self.ThemeList:
			self.setTheme("Dracula")
		return self.UserSetting().value("BasicInfo/Theme")
	
	def setTheme(self, Theme:str):
		self.setHue(-1)
		self.setSaturation(0.5)
		self.setLuminance(0.5)
		self.setContrast(0.5)
		self.setReverse(False)
		self.UserSetting().setValue("BasicInfo/Theme",Theme)
	
	def Hue(self):
		if self.UserSetting().value("BasicInfo/Hue")==None:
			self.setHue(-1)
		else:
			Hue=float(self.UserSetting().value("BasicInfo/Hue"))
			if not 0<=Hue<=1 and Hue!=-1:
				self.setHue(-1)
		return float(self.UserSetting().value("BasicInfo/Hue"))
	
	def setHue(self, hue:float):
		self.UserSetting().setValue("BasicInfo/Hue",str(hue))

	def Saturation(self):
		if self.UserSetting().value("BasicInfo/Saturation")==None:
			self.setSaturation(0.5)
		else:
			Saturation=float(self.UserSetting().value("BasicInfo/Saturation"))
			if not 0<=Saturation<=1:
				self.setSaturation(0.5)
		return float(self.UserSetting().value("BasicInfo/Saturation"))
	
	def setSaturation(self, saturation:float):
		self.UserSetting().setValue("BasicInfo/Saturation",str(saturation))

	def Luminance(self):
		if self.UserSetting().value("BasicInfo/Luminance")==None:
			self.setLuminance(0.5)
		else:
			Luminance=float(self.UserSetting().value("BasicInfo/Luminance"))
			if not 0<=Luminance<=1:
				self.setLuminance(0.5)
		return float(self.UserSetting().value("BasicInfo/Luminance"))
	
	def setLuminance(self, luminance:float):
		self.UserSetting().setValue("BasicInfo/Luminance",str(luminance))
	
	def Contrast(self):
		if self.UserSetting().value("BasicInfo/Contrast")==None:
			self.setContrast(0.5)
		else:
			Contrast=float(self.UserSetting().value("BasicInfo/Contrast"))
			if not 0<=Contrast<=1:
				self.setContrast(0.5)
		return float(self.UserSetting().value("BasicInfo/Contrast"))
	
	def setContrast(self, contrast:float):
		self.UserSetting().setValue("BasicInfo/Contrast",str(contrast))

	def Reverse(self):
		if self.UserSetting().value("BasicInfo/Reverse")==None:
			self.setReverse(False)
		
		Reverse = self.UserSetting().value("BasicInfo/Reverse")
		Reverse = True if Reverse=="True" else False
		return Reverse

	def setReverse(self, reverse:bool):
		self.UserSetting().setValue("BasicInfo/Reverse",str(reverse))

	def Language(self):
		return self.UserSetting().value("BasicInfo/Language")
	
	def setLanguage(self, Language:str):
		self.UserSetting().setValue("BasicInfo/Language",Language)
	
	def Country(self):
		return self.UserSetting().value("BasicInfo/Country")
	
	def setCountry(self, Country:str):
		self.UserSetting().setValue("BasicInfo/Country",Country)

	def setApplicationName(self, str):
		super().setApplicationName(str)
		self.UserSetting().setValue("MetaData/ApplicationName",self.applicationName())
	
	def setApplicationVersion(self, str):
		super().setApplicationVersion(str)
		self.UserSetting().setValue("MetaData/ApplicationVersion",self.applicationVersion())
	
	def setOrganizationName(self, str):
		super().setOrganizationName(str)
		self.UserSetting().setValue("MetaData/OrganizationName",self.organizationName())
	
	def setOrganizationDomain(self, str):
		super().setOrganizationDomain(str)
		self.UserSetting().setValue("MetaData/OrganizationDomain",self.organizationDomain())

	def UserSetting(self):
		return self.__UserSetting

	def setQuitOnClickX(self, bool):
		self.__quit_on_click_x=bool
	
	def isQuitOnClickX(self):
		return self.__quit_on_click_x
	
	def isLoginEnable(self):
		return self.__LoginEnable
	
	def setLoginEnable(self, bool=True):
		self.__LoginEnable=bool
	
	def setDataDir(self, new_dir:str):
		if os.path.exists(new_dir):
			old_dir=self.DataDir()
			try:
				for file in self.DataList():		
					old_file_dir=os.path.join(old_dir,file)
					file_dst=os.path.join(new_dir,file)
					shutil.move(old_file_dir,file_dst)
			except Exception as e:
				DTFrame.DTMessageBox(None,"Error",e,DTIcon.Error())
			
			self.__data_dir=new_dir
			if self.isLoginEnable():
				self.UserSetting().setValue("BasicInfo/DataDir",Fernet_Encrypt(self.password(),self.__data_dir))
			else:
				self.UserSetting().setValue("BasicInfo/DataDir",self.__data_dir)
		else:
			DTFrame.DTMessageBox(None,"Error","Data Dir does not exsit!",DTIcon.Error())
	
	def DataDir(self):
		if self.UserSetting().value("BasicInfo/DataDir")==None:
			self.__data_dir=os.getcwd()
			if self.isLoginEnable():
				self.UserSetting().setValue("BasicInfo/DataDir",Fernet_Encrypt(self.password(),self.__data_dir))
			else:
				self.UserSetting().setValue("BasicInfo/DataDir",self.__data_dir)
		return self.__data_dir

	def setDataList(self, data_list:list):
		"""程序中对app设置，data文件列表（应该是DataDir下的文件，与load、save、backup相关）

		Args:
			data_list (list): data文件列表
		"""
		self.__DataList=data_list
	
	def DataList(self):
		return self.__DataList
	
	def isBackupEnable(self):
		return self.__BackupEnable
	
	def setBackupEnable(self, bool=True):
		"""设置是否开启备份功能，备份文件的列表即为main中setDataList设置的文件列表

		Args:
			bool (bool, optional): Defaults to True.
		"""
		if self.DataList()==[] and bool==True:
			raise("Please setDataList before setBackEnable.")
		else:
			self.__BackupEnable=bool
	
	def setBackupDst(self, dst:str):
		"""设置面板中设置备份地址

		Args:
			dst (str): 备份的目的地
		"""
		if self.isBackupEnable():
			if os.path.exists(dst):
				self.__backup_dst=dst
				if self.isLoginEnable():
					self.UserSetting().setValue("BasicInfo/BackupDst",Fernet_Encrypt(self.password(),self.__backup_dst))
				else:
					self.UserSetting().setValue("BasicInfo/BackupDst",self.__backup_dst)
			else:
				DTFrame.DTMessageBox(None,"Error","Backup Dst does not exsit!",DTIcon.Error())
		else:
			raise("Please setBackupEnable before setBackupDst.")
	
	def BackupDst(self):
		return self.__backup_dst

	def password(self):
		return self.__password
	
	def setPassword(self, password):
		self.__password=password
		self.UserSetting().setValue("BasicInfo/Password",Fernet_Encrypt(self.password(),self.password()))

	def author(self):
		return self.UserSetting().value("MetaData/Author")

	def setAuthor(self, author):
		self.UserSetting().setValue("MetaData/Author",author)
	
	def contact(self):
		return self.UserSetting().value("MetaData/Contact")

	def setContact(self, contact):
		self.UserSetting().setValue("MetaData/Contact",contact)
	
	def setMainSession(self, mainsession: DTSession.DTMainSession):
		self.__mainsession=mainsession
	
	def showMessage(self, title:str, msg:str, icon:QIcon, msecs:int=1000, clicked_slot=None):
		self.TrayIcon.showMessage(title, msg, icon, msecs)
		try:
			self.TrayIcon.messageClicked.disconnect() # 取消所有的信号槽，如果之前没有连接信号，这里会报错，所以加了个try
		except:
			pass
		self.TrayIcon.messageClicked.connect(clicked_slot)

	def restart(self):
		self.__mainsession.saveWindowStatus()
		self.__mainsession.saveData()
		self.__mainsession.close()
		self.exit()
		if self.isLoginEnable()==True:
			QProcess.startDetached(sys.executable, sys.argv+[str(Fernet_Encrypt("9961",self.password())), "9961"])
		else:
			QProcess.startDetached(sys.executable, sys.argv+["9961"])
	
	def __loginIn(self):
		dlg=DTSession.DTLoginSession(self.UserSetting().value("BasicInfo/Password"))
		if dlg.exec_()==0:
			self.quit()
			sys.exit()
		else:
			self.setPassword(dlg.input_password)
	
	def __loadEncryptedData(self):
		if self.isLoginEnable():
			self.__data_dir=Fernet_Decrypt(self.password(),self.UserSetting().value("BasicInfo/DataDir"))
			if self.__data_dir==False:
				self.__data_dir=None
			self.__backup_dst=Fernet_Decrypt(self.password(),self.UserSetting().value("BasicInfo/BackupDst"))
			if self.__backup_dst==False:
				self.__backup_dst=None
		else:
			self.__data_dir=self.UserSetting().value("BasicInfo/DataDir")
			self.__backup_dst=self.UserSetting().value("BasicInfo/BackupDst")
			
	def debugRun(self,password,loginEnable,show=True):
		
		self.setLoginEnable(loginEnable)
		
		# Restart
		if self.arguments()[-1]=="9961":
			if self.isLoginEnable()==True:
				# print("Args:",self.arguments())
				self.setPassword(Fernet_Decrypt("9961", eval(self.arguments()[-2])))
				# print("Restart Password:",self.password())

			self.__loadEncryptedData()
			
			self.__mainsession.initialize()
			self.__mainsession.show()

			self.initializeTrayIcon()
			self.exec_()
		
		# Normal
		else:
			self.setPassword(password)

			self.__loadEncryptedData()

			self.__mainsession.initialize()
			if show==True:
				self.__mainsession.show()

			self.initializeTrayIcon()
			self.exec_()

	def run(self,show=True):
		"""如果没有密码加密，需要setLoginEnable(False)
		"""

		# Restart
		if self.arguments()[-1]=="9961":
			if self.isLoginEnable()==True:
				# print("Args:",self.arguments())
				self.setPassword(Fernet_Decrypt("9961", eval(self.arguments()[-2])))
				# print("Restart Password:",self.password())

			self.__loadEncryptedData()

			self.__mainsession.initialize()
			self.__mainsession.show()

			self.initializeTrayIcon()
			self.exec_()
		
		# Normal
		else:
			if self.isLoginEnable()==True:
				self.__loginIn()
			
			self.__loadEncryptedData()
			
			self.__mainsession.initialize()
			if show==True:
				self.__mainsession.show()
			
			self.initializeTrayIcon()
			self.exec_()


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'Ui_DTLogin.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

import DTPySide.DT_rc

class Ui_DTLogin(object):
    def setupUi(self, DTLogin):
        if not DTLogin.objectName():
            DTLogin.setObjectName(u"DTLogin")
        DTLogin.resize(221, 50)
        self.horizontalLayout = QHBoxLayout(DTLogin)
        self.horizontalLayout.setSpacing(7)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(6, 10, 0, 15)
        self.label_lock = QLabel(DTLogin)
        self.label_lock.setObjectName(u"label_lock")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_lock.sizePolicy().hasHeightForWidth())
        self.label_lock.setSizePolicy(sizePolicy)
        self.label_lock.setMinimumSize(QSize(28, 28))
        self.label_lock.setMaximumSize(QSize(28, 28))
        self.label_lock.setScaledContents(True)

        self.horizontalLayout.addWidget(self.label_lock)

        self.lineEdit = QLineEdit(DTLogin)
        self.lineEdit.setObjectName(u"lineEdit")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.lineEdit.sizePolicy().hasHeightForWidth())
        self.lineEdit.setSizePolicy(sizePolicy1)
        self.lineEdit.setMinimumSize(QSize(180, 26))
        self.lineEdit.setMaximumSize(QSize(16777215, 28))
        font = QFont()
        font.setPointSize(12)
        self.lineEdit.setFont(font)
        self.lineEdit.setEchoMode(QLineEdit.Password)

        self.horizontalLayout.addWidget(self.lineEdit)


        self.retranslateUi(DTLogin)

        QMetaObject.connectSlotsByName(DTLogin)
    # setupUi

    def retranslateUi(self, DTLogin):
        DTLogin.setWindowTitle(QCoreApplication.translate("DTLogin", u"DTLogin", None))
        self.label_lock.setText("")
        self.lineEdit.setText("")
    # retranslateUi




import os

def deepin(base):
	
	base=os.path.abspath(base)
	os.chdir(base)

	for i in os.listdir("./"):
		if os.path.isdir(i):
			deepin(i)
			os.chdir(base)
		else:
			ext=os.path.splitext(i)[1]
			if ext==".ui" or ext==".py":
				ui_list.append(" "+os.path.abspath(i))

ui_list=[]
base=os.getcwd()
deepin(base)
os.system("pyside2-lupdate %s -ts en.ts"%("".join(ui_list)))


from __future__ import annotations
from DTPySide import *

class DTStyleSheet(str):
	def __new__(cls, color_list, window_effect:str, font:QFont):
		
		# font_size=18
		font_family=font.family()

		DEEPDARK,BACKGROUND,SOFTDARK,DIM,PRESSED,FOCUSED,TEXT=color_list
		
		ICONCOLOR=QIcon.themeName()

		stylesheet=""

		if window_effect=="Normal":
			stylesheet+=f"""
				* {{
					background: {BACKGROUND};
				}}
			"""
			pass
		else:
			stylesheet+=f"""
				* {{
					background: transparent;
				}}

				QDialog{{
					background-color: {BACKGROUND};
				}}
			"""
		

		stylesheet+=f"""

		QWidget
		{{
			color: {TEXT};
			font-family: {font_family};
			font-size: 14pt;
			selection-background-color: {FOCUSED};
			selection-color: {TEXT};
		}}
		QWidget::item:active, QWidget::item:!active
		{{
			color: {TEXT};
		}}
		
		QWidget:disabled
		{{
			color: {DIM};
		}}

		QDialog {{
			border: 2px solid {DEEPDARK};
			border-radius: 1px;
		}}

		#DTMainWindow #statusBar{{
			background-color: {SOFTDARK};
			font-family: "微软雅黑";
			font-size: 9pt;
		}}

		/* Login */
		DTLoginSession QLineEdit {{
			font-size: 12pt;
		}}

		DTLoginSession DTTitleBar QLabel {{
			font-family: "Segoe UI";
			font-size: 20pt;
		}}

		DTLoginSession DTTitleBar #title_icon{{
			icon-size: 36px;
			max-height: 36px;
			min-height: 36px;
			min-width: 36px;
			max-width: 36px;
		}}
		
		/* TitleBar */
		DTTitleBar QLabel {{
			font-family: "Segoe UI";
			font-size: 20px;
		}}
		
		DTTitleBar QPushButton {{
			border: none;
			background-color: transparent;
			max-height: 24px;
			min-height: 24px;
			min-width: 24px;
			max-width: 24px;
		}}

		DTTitleBar QPushButton:hover{{
			background-color: {SOFTDARK};
			border-style: solid;
			border-radius: 4px;
		}}
		DTTitleBar QPushButton:pressed {{
			background-color: {DIM};
			border-style: solid;
			border-radius: 4px;
		}}

		DTTitleBar #title_icon{{
			icon-size: 26px;
			max-height: 26px;
			min-height: 26px;
			min-width: 26px;
			max-width: 26px;
		}}
		DTTitleBar #title_icon:hover{{
			background-color: transparent;
		}}
		DTTitleBar #title_icon:pressed {{
			background-color: transparent;
		}}
		
		DTTitleBar #btn_close:hover{{
			background-color: rgb(232, 17, 35);
			border-style: solid;
			border-radius: 4px;
		}}
		DTTitleBar #btn_close:pressed {{
			background-color: rgb(241, 112, 122);
			border-style: solid;
			border-radius: 4px;
		}}
		DTTitleBar * {{
			background-color: {DEEPDARK};
		}}
		DTTitleBar {{
			background-color: {DEEPDARK};
		}}

		/* ---------------------------------------- */

		QFrame{{
			color: {TEXT};
		}}
		QMainWindow::separator{{
			border: 1px solid {DEEPDARK};
			width: 4px;
			height: 4px;
		}}
		QMainWindow::separator:hover{{
			background: {FOCUSED};
		}}

		QSplitter{{
			background: transparent;
		}}

		QSplitter::handle{{
			border: 1px solid transparent;
			margin: 2px;
			border-radius: 6px;
		}}
		QSplitter::handle:horizontal{{
			width: 6px;
			image: url(:/icons/{ICONCOLOR}/more-vertical.svg);
		}}
		QSplitter::handle:vertical{{
			height: 6px;
			image: url(:/icons/{ICONCOLOR}/more-horizontal.svg);
		}}
		QSplitter::handle:hover{{
			background: {SOFTDARK};
		}}
		

		QSizeGrip{{
			background-color: none;
		}}

		/* MenuBar Menu */
		QMenuBar {{
			background-color: {BACKGROUND};
			font-size: 10pt;
			spacing: 1px;
			border-bottom: 1px solid {DEEPDARK};
		}}

		QMenuBar::item{{
			background: transparent;
			font-size: 10pt;
			padding: 1px 8px;
		}}

		QMenuBar::item:selected{{
			background: {FOCUSED};
			border: 1px solid {DEEPDARK};
		}}

		QMenuBar::item:pressed{{
			background: {PRESSED};
			border: 1px solid {DEEPDARK};
			padding-top: 4px;
		}}

		QMenu {{
			font-family: "微软雅黑";
			font-size: 10pt;
			background-color: {BACKGROUND};
			margin: 1px;
			padding: 1px;
		}}

		QMenu::item{{
			font-family: "微软雅黑";
			font-size: 10pt;
			padding: 2px 25px 2px 15px;
			border: 1px solid transparent;
			margin: 1px;
		}}
		QMenu::item:disabled{{
			color: {DIM};
		}}
		QMenu::icon {{
			width: 15px;
			height: 15px;
			padding:5px;
			border-right: 1px inset {DEEPDARK};
		}}

		QMenu::item:selected {{
			border-color: {DEEPDARK};
			background: {FOCUSED};
		}}

		QMenu::separator {{
			height: 2px;
			background: {DEEPDARK};
		}}

		QMenu::indicator {{/*checked 的√*/
			width: 13px;
			height: 13px;
			padding:2px;
		}}
		QMenu::icon:checked {{ /* appearance of a 'checked' icon */
			background: {FOCUSED};
			border: 1px inset {DEEPDARK};
			border-radius: 3px;
			padding: 2px;
		}}


		/* ToolBar */

		QToolBar {{
			background: {BACKGROUND};
			spacing: 1px;
			padding: 1px;
			border-bottom: 1px solid {DEEPDARK};
		}}


		/* Label */

		QLabel {{
			background: transparent;
		}}

		QToolTip {{
			background: {SOFTDARK};
			color: {TEXT}
		}}


		/* TextBox */
		
		QLineEdit {{
			background: {SOFTDARK};
			font-size: 12pt;
			border-radius: 6px;
			height: 26px;
		}}


		QLineEdit:focus{{
			border: 1px solid;
			border-color: {FOCUSED};
		}}

		QLineEdit:read-only {{
			color: {DIM};
		}}

		QTextEdit, QPlainTextEdit, QTextBrowser{{
			border-radius: 6px;
			font-size: 14pt;
			background-color: {SOFTDARK};
		}}
		QTextEdit:focus, QPlainTextEdit:focus, QTextBrowser:focus{{
			border: 1px solid;
			border-color: {FOCUSED};
		}}

		/* Button */


		QPushButton {{
			border: 1px solid transparent;
			border-radius: 3px;
			background-color: {SOFTDARK};
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
			min-height: 26px;
		}}

		QPushButton:hover{{
			background-color: {FOCUSED};
			border-color: {FOCUSED};
		}}

		QPushButton:pressed
		{{
			background-color: {PRESSED};
			border-color: {FOCUSED};
		}}

		QPushButton:focus, QPushButton:default {{
			border-color: {FOCUSED};
		}}

		QDialogButtonBox QPushButton{{
			min-width: 72px;
		}}
		
		QToolButton {{
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
			height: 27px;
		}}
		QToolButton,QToolButton:unchecked {{
			/* ToolBar里的按钮和带下拉菜单的按钮 */
			border: 1px solid transparent;
			border-radius: 3px;
			background-color: {SOFTDARK};
			margin: 1px;
		}}
		QToolButton:checked{{
			background-color: {FOCUSED};
			border-color: {DEEPDARK};
		}}
		QToolButton:hover{{
			background-color: {FOCUSED};
			border-color: {DEEPDARK};
		}}

		QToolButton:pressed,QToolButton:checked:hover{{
			background-color: {PRESSED};
			border-color: {DEEPDARK};
		}}
		QToolButton:checked:pressed{{
			background-color: {FOCUSED};
		}}

		/* only for MenuButtonPopup */
		QToolButton[popupMode="1"]{{
			padding-left: 1px;
			padding-right: 15px;
			border: 1px solid {DEEPDARK};
			min-height: 15px;
			background-color: {SOFTDARK};
		}}
		QToolButton[popupMode="1"]:hover{{
			background-color: {FOCUSED};
			border-color: {DEEPDARK};
		}}
		QToolButton[popupMode="1"]:pressed{{
			border-width: 1px;
			background-color: {PRESSED};
			border-color: {DEEPDARK};
		}}
		QToolButton::menu-button {{
			border: 1px solid {DEEPDARK};
			border-top-right-radius: 2px;
			border-bottom-right-radius: 2px;
			width: 16px;
		}}

		QRadioButton{{
			background: transparent;
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
		}}
		QRadioButton::indicator {{
			width: 10px;
			height: 10px;
			border: 2px solid {TEXT};
			border-radius: 7px;
		}}
		QRadioButton::indicator:checked {{
			background: {FOCUSED};
		}}
		QRadioButton::indicator:unchecked {{
			background: transparent;
		}}

		QCheckBox{{
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
			background: transparent;
		}}
		QCheckBox::indicator {{
			width: 10px;
			height: 10px;
			border: 2px solid {TEXT};
			border-radius: 7px;
		}}
		QCheckBox::indicator:checked {{
			background-color: {FOCUSED};
		}}
		QCheckBox::indicator:unchecked {{
			background-color: transparent;
		}}
		QCheckBox::indicator:indeterminate {{
			background-color: {FOCUSED};
			border-radius: 2px;
		}}

		

		/* ComboBox */

		QComboBox{{
			background-color: {SOFTDARK};
			border-radius: 6px;
			font-size: 12pt;
			height: 26px;
		}}
		QComboBox:focus{{
			border: 1px solid;
			border-color: {FOCUSED};
		}}
		QComboBox QAbstractItemView{{
			background-color: {DEEPDARK};
		}}
		QComboBox::drop-down{{
			background-color: {SOFTDARK};
			border: 1px solid transparent;
			border-top-right-radius: 6px;
			border-bottom-right-radius: 6px;
			subcontrol-origin: padding;
			subcontrol-position: top right;
			width: 18px;
			margin: 1px;
		}}
		QComboBox::drop-down:pressed{{
			background-color: {FOCUSED};
		}}
		QComboBox::down-arrow{{
			image: url(:/icons/{ICONCOLOR}/chevron-down.svg);
		}}
		

		/* SpinBox DateTime */

		QSpinBox, QDoubleSpinBox, QDateEdit, QTimeEdit, QDateTimeEdit{{
			font-family: {font_family}; /*"微软雅黑";*/
			background-color: {SOFTDARK};
			font-size: 12pt;
			border-radius: 6px;
			height: 26px;
		}}
		QSpinBox:focus, QDoubleSpinBox:focus, QDateEdit:focus, QTimeEdit:focus, QDateTimeEdit:focus{{
			border: 1px solid {FOCUSED};
		}}

		QSpinBox::up-button, QDoubleSpinBox::up-button, QDateEdit::up-button, QTimeEdit::up-button, QDateTimeEdit::up-button{{
			background-color: {SOFTDARK};
			subcontrol-origin: border;
			subcontrol-position: top right;
			width: 18px;
			border: 2px ridge transparent;
			border-top-right-radius: 6px;
			margin-top: 1px;
			margin-right: 1px;
		}}
		QSpinBox::up-button:pressed, QDoubleSpinBox::up-button:pressed, QDateEdit::up-button:pressed, QTimeEdit::up-button:pressed, QDateTimeEdit::up-button:pressed {{
			background-color: {FOCUSED};
		}}
		QSpinBox::up-arrow, QDoubleSpinBox::up-arrow, QDateEdit::up-arrow, QTimeEdit::up-arrow, QDateTimeEdit::up-arrow{{
			image: url(:/icons/{ICONCOLOR}/chevron-up.svg);
		}}

		QSpinBox::down-button, QDoubleSpinBox::down-button, QDateEdit::down-button, QTimeEdit::down-button, QDateTimeEdit::down-button{{
			background-color: {SOFTDARK};
			subcontrol-origin: border;
			subcontrol-position: bottom right;
			width: 18px;
			border: 2px ridge transparent;
			border-bottom-right-radius: 6px;
			margin-bottom: 1px;
			margin-right: 1px;
		}}
		QSpinBox::down-button:pressed, QDoubleSpinBox::down-button:pressed, QDateEdit::down-button:pressed, QTimeEdit::down-button:pressed, QDateTimeEdit::down-button:pressed {{
			background-color: {FOCUSED};
		}}
		QSpinBox::down-arrow, QDoubleSpinBox::down-arrow, QDateEdit::down-arrow, QTimeEdit::down-arrow, QDateTimeEdit::down-arrow{{
			image: url(:/icons/{ICONCOLOR}/chevron-down.svg);
		}}
		


		/* Slider ProgressBar */

		QProgressBar {{
			background-color: {SOFTDARK};
			border-radius: 6px;
			text-align: center;
		}}

		QProgressBar::chunk {{
			border-radius: 6px;
			background-color: {FOCUSED};
		}}

		QSlider{{
			background: transparent;
			min-height: 14px;
			min-width: 14px;
		}}
		
		QSlider::handle {{
			background: {FOCUSED};
			border: 1px solid {DEEPDARK};
			border-radius: 6px;
		}}

		QSlider::groove:horizontal {{
			height: 12px;
		}}
		QSlider::handle:horizontal{{
			width: 12px;
			height: 14px;
			margin: -7px;
		}}

		QSlider::groove:vertical{{
			width: 12px;
		}}
		QSlider::handle:vertical{{
			width: 14px;
			height: 12px;
			margin: -7px;
		}}
		
		QSlider::add-page{{
			/*还没有滑上去的地方*/
			background:{SOFTDARK};
		}}
		QSlider::sub-page{{
			/*已经划过的从地方*/
			background: {FOCUSED};
		}}


		/* ScrollBar */

		QScrollBar{{
			width: 16px;
			height: 16px;
			background-color: transparent;
			padding: 1px;
		}}
		QScrollBar::handle{{
			min-height: 30px;
		}}
		QScrollBar::handle{{
			border-radius: 5px;
			background: {FOCUSED};
		}}
		QScrollBar::handle:hover {{
			background: {PRESSED};
		}}
		QScrollBar::add-line, QScrollBar::sub-line, QScrollBar::add-page, QScrollBar::sub-page {{
			background: transparent;
			height: 0px;
		}}
		QScrollArea{{
			border: none;
		}}
		QAbstractScrollArea::corner {{
			background: transparent;
		}}

		/* DockWidget*/

		QDockWidget, QDockWidget > QWidget /*not work*/
		{{
			border-color: {DEEPDARK}; /*qt bug*/
			background: transparent;
		}}
		QDockWidget::title {{
			border-bottom: 1px solid {DEEPDARK};
			text-align: left;
			padding: 6px;
		}}


		/* GroupBox */

		QGroupBox {{
			border: 2px solid {DEEPDARK};
			border-radius: 6px;
			margin-top:1em;
		}}
		QGroupBox::title {{
			subcontrol-origin: margin;
			subcontrol-position: top left;
		}}

		/* ToolBox */

		QToolBox{{
			border: 1px solid transparent;
		}}
		QToolBox::tab {{
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
			background: {SOFTDARK};
			border-color: transparent;
			border-radius: 6px;
			padding-left: 4px;
		}}
		QToolBox::tab:hover {{
			background-color: {FOCUSED};
			border-color: transparent;
		}}
		QToolBox::tab:pressed {{
			background-color: {PRESSED};
			border-color: transparent;
		}}
		QToolBox::tab:selected {{
			font-weight: bold;
			border-color: {DEEPDARK};
		}}


		/* TabWidget */

		QTabWidget{{
			margin-top:10px;
		}}
		QTabWidget::pane{{
			margin-top:5px;
			border: none;
		}}
		QTabWidget::tab-bar {{
			left: 0px;
		}}
		QTabBar::tab {{
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
			background: {SOFTDARK};
			border-color: transparent;
			border-radius: 4px;
			left: -8px;
			margin-left: 8px;
		}}
		QTabBar::tab:hover {{
			background: {FOCUSED};
		}}
		QTabBar::tab:selected {{
			background: {FOCUSED};
		}}
		QTabBar::tab:pressed {{
			background: {PRESSED};
		}}
		
		/*别显示左右的按钮了，滚轮滚就行了*/
		QTabBar::scroller {{
			width: 0px;
		}}


		/* QHeaderView for list table */

		QAbstractItemView, QAbstractItemView QLineEdit {{
			font-family: {font_family};
			font-size: 12pt;
			background-color: {SOFTDARK};
		}}
		
		QHeaderView {{
			border: none;
			font-size: 10pt;
		}}
		QHeaderView::section, QTableCornerButton::section {{
			/*设置表头属性*/ /*表格左上角小框框*/
			background-color: {DIM};
			padding-top: 5px;
			border-right: 1px solid {DEEPDARK};
			border-bottom: 1px solid {DEEPDARK};
			border-radius: 0px;
		}}
		QHeaderView::section:hover, QTableCornerButton::section:hover{{
			background-color: {DIM};
		}}
		QHeaderView::section:pressed{{
			background-color: {DIM};
		}}
		QHeaderView::section:checked {{
			background-color: {DIM};
		}}


		/* QTableWidget */

		QTableWidget, QTableView
		{{
			font-size: 12pt;
			gridline-color: {DEEPDARK}; /*表格中的网格线条颜色*/
			border-color: transparent; /*边框线的宽度、颜色*/
			border-radius: 6px;
		}}
		

		QTableWidget QComboBox{{
			margin: 2px;
			border: none;
		}}

		QListView, QTreeView, QTableView {{
			font-size: 12pt;
			border-color: transparent; /*边框线的宽度、颜色*/
			border-radius: 6px;
		}}
		

		QListView::item, QTreeView::item, QTableView::item{{
			font-size: 12pt;
			background: {DIM};
			outline-style: none;
			border-left: 2px solid {PRESSED};
		}}
		QListView::item:selected , QTreeView::item:selected, QTableView::item:selected {{
			background: {FOCUSED};
			color: {TEXT};
		}}

		QListView::item:hover, QTreeView::item:hover, QTableView::item:hover {{
			background: {FOCUSED};
		}}
		

		QTreeView::branch{{
			
		}}
		QTreeView::branch:selected{{
			
		}}


		QFontDialog {{
			min-width: 900px;
			min-height: 700px;
		}}
		QFontDialog * {{
			font-size: 17px;
		}}
		
		/* QCalendar */

		/* header row */
		QCalendarWidget QWidget {{
			font-family: {font_family}; /*"微软雅黑";*/
			font-size: 12pt;
			alternate-background-color: {SOFTDARK};
		}}
		
		/* normal days */
		QCalendarWidget QAbstractItemView:enabled
		{{
			background-color: {BACKGROUND};
		}}
		
		"""
		# QStatusBar{{
		# 	background: transparent;
		# 	border-top:1px solid {DEEPDARK};
		# }}

		# QStatusBar::item {{
		# 	margin: 2px 0;
		# 	border-left: 1px solid {DEEPDARK};
		# }}

		# QStatusBar QLabel{{
		# 	background: transparent;
		# 	margin: 0  2px;
		# }}

		# QStatusBar QPushButton{{
		# 	background: transparent;
		# 	margin: 0  2px;
		# }}

		# QStatusBar QPushButton:hover{{
		# 	background: {FOCUSED};
		# 	margin: 0  2px;
		# }}
		return str.__new__(cls,stylesheet)



from __future__ import annotations
from DTPySide import *

class DTLine(QFrame):
	def __init__(self, parent, shape: QFrame.Shape = QFrame.HLine, shadow: QFrame.Shadow = QFrame.Sunken) -> None:
		super().__init__(parent)
		self.setFrameShape(shape)
		self.setFrameShadow(shadow)


from __future__ import annotations
from DTPySide import *

# Mainwindow
from DTPySide.DTFrame.Ui_DTMainWindow import Ui_DTMainWindow
class DTMainWindow(Ui_DTMainWindow,DTWidget.DTWindow):
	"DTMainWindow骨架，通过setCentralWidget来放入主功能区"
	
	def __init__(self, app:DTAPP):
		super().__init__(app)
		self.setupUi(self)
		self.app=app
		self._MainMenu=QMenu(self)
	
	def initialize(self):
		self.initializeWindow()
		self.initializeSignal()
		self.initializeMenu()

	def initializeWindow(self):
		self.TitleBar.setFull(True)
		self.TitleBar.updateWindowIcon()
		self.setWindowTitle(self.app.applicationName())

		self.actionSetting.setIcon(IconFromCurrentTheme("settings.svg"))
		self.actionExit.setIcon(IconFromCurrentTheme("log-out.svg"))
		self.actionMaximize_Window.setIcon(IconFromCurrentTheme("window-maximize.svg"))
		self.actionNormalize_Window.setIcon(IconFromCurrentTheme("window-restore.svg"))
		self.actionMinimize_Window.setIcon(IconFromCurrentTheme("window-minimize.svg"))
		self.actionWindow_Toggle_Stay_on_Top.setIcon(IconFromCurrentTheme("arrow-up.svg"))
		self.actionWindow_Toggle_Fullscreen.setIcon(IconFromCurrentTheme("maximize.svg"))
		self.actionWindow_Toggle_Fullscreen.setIcon(IconFromCurrentTheme("maximize.svg"))
		self.actionAbout.setIcon(IconFromCurrentTheme("book.svg"))
		self.actionBoss_Key.setIcon(IconFromCurrentTheme("users.svg"))
		self.actionBoss_Key.setIcon(IconFromCurrentTheme("users.svg"))
		self.actionBackup.setIcon(IconFromCurrentTheme("cloud.svg"))
	
	def initializeSignal(self):	
		"""主窗体的action向function链接
		注意：需要拥有全局快捷键的action，需要addAction
		"""
		
		self.TitleBar.btn_close.clicked.connect(self.close)
		
		self.actionWindow_Toggle_Fullscreen.triggered.connect(self.windowToggleFullscreen)
		self.addAction(self.actionWindow_Toggle_Fullscreen)

		self.actionWindow_Toggle_Stay_on_Top.triggered.connect(self.windowToggleStayonTop)
		self.addAction(self.actionWindow_Toggle_Stay_on_Top)

		self.actionNormalize_Window.triggered.connect(self.windowShowNormal)
		self.actionMaximize_Window.triggered.connect(self.showMaximized)
		self.actionMinimize_Window.triggered.connect(self.showMinimized)

		self.actionExit.triggered.connect(self.close)
		self.addAction(self.actionExit)

	def initializeMenu(self):
		"制定menu"
		
		self.__menu_view=QMenu(QCoreApplication.translate("DTMainWindow","View"),self) #这里不能用self.tr，因为如果被继承了，这里的self将会是子类，而在qm翻译文件的context中记录的是DTMainSWindow的翻译
		self.__menu_view.setIcon(IconFromCurrentTheme("eye.svg"))
		self.__menu_view.addAction(self.actionWindow_Toggle_Stay_on_Top)
		self.__menu_view.addAction(self.actionWindow_Toggle_Fullscreen)
		self.__menu_view.addAction(self.actionNormalize_Window)
		self.__menu_view.addAction(self.actionMinimize_Window)
		self.__menu_view.addAction(self.actionMaximize_Window)
		self.addMenuToMainMenu(self.__menu_view)

		self._MainMenu.addAction(self.actionExit)
	
	def MainMenu(self):
		return self._MainMenu
	
	def addSeparatorToMainMenu(self):
		self._MainMenu.addSeparator()

	def addActionToMainMenu(self,action):
		self._MainMenu.addAction(action)
	
	def insertActionToMainMenu(self,fore_action,action):
		self._MainMenu.insertAction(fore_action,action)
	
	def addMenuToMainMenu(self,menu):
		self._MainMenu.addMenu(menu)
	
	def insertMenuToMainMenu(self,fore_action,menu):
		self._MainMenu.insertMenu(fore_action,menu)

	def windowShowNormal(self):
		self.showNormal()
		self.resize(self.minimumWidth(),self.minimumHeight())
		MoveToCenterOfScreen(self)

	def windowToggleStayonTop(self):
		"切换主窗体的置顶与否"

		if bool(self.windowFlags() & Qt.WindowStaysOnTopHint):
			self.setWindowFlag(Qt.WindowStaysOnTopHint,False)
			# 貌似设置了置顶后缩放就侦测不到了，手动重置一下
			self.reInitialize()
		else:
			self.setWindowFlag(Qt.WindowStaysOnTopHint,True)
		
		if self.isFullScreen():
			self.showFullScreen()
		else:
			self.showNormal()
		
	
	def windowToggleFullscreen(self):
		"切换主窗体的全屏与否"
		
		if self.isFullScreen():
			self.showNormal()
			self.TitleBar.show()
		else:
			self.showFullScreen()
			self.TitleBar.hide()

	def setWindowTitle(self,title):
		super().setWindowTitle(title)
		try:
			self.TitleBar.setWindowTitle(title)
		except:
			pass
	
	def setStatusTip(self, tip:str):
		self.statusBar.setText(tip)
	
	def password(self):
		return self.app.password()
	
	def setPassword(self,password):
		self.app.setPassword(password)
	
	def UserSetting(self):
		return self.app.UserSetting()

	def setCentralWidget(self,widget):
		# 奶奶的，这里本来偷懒，在ui文件中用了一个QWidget占位
		# 这里就写了self.centralwidget=widget直接暴力替换
		# 结果把TitleBar区域的判定搞坏掉了
		# TitleBar的mousePressEvent、mouseMoveEvent等事件在左上角的一个区域内都侦测不到了
		# 花了我将近两个小时才捉出这只虫子……
		
		self.centralwidget.addWidget(widget,Qt.AlignCenter)


from DTPySide import *

from DTPySide.demo.demoModule.Ui_DemoCentralWidget2 import Ui_DemoCentralWidget2
class DemoCentralWidget2(Ui_DemoCentralWidget2,QWidget):
    def __init__(self,Headquarter):
        super().__init__(Headquarter)
        self.setupUi(self)
        
        self.homelabel.setText("Your password is "+Headquarter.password())


from __future__ import annotations
from DTPySide import *

class DTHorizontalTabel(QTableWidget):

	sortReset=Signal()
	
	def __init__(self, parent):
		super().__init__(parent=parent)

		self.sort_times=0
		
		# 不知道为什么self.setSortingEnabled(True)在手动排序时会吞字
		# 下面这样设置一下就没问题……
		header = self.horizontalHeader()
		header.setSortIndicatorShown(True)
		header.sectionClicked.connect(self.sort)
		
		self.setEditTriggers(QAbstractItemView.NoEditTriggers)
		
		self.horizontalScrollBar().setVisible(False)
		# self.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
		# self.horizontalScrollBar().setSingleStep(4)
		# self.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)
		# self.verticalScrollBar().setSingleStep(4)

		self.horizontalHeader().setStretchLastSection(True)
		self.horizontalHeader().setMinimumSectionSize(80)
		self.horizontalHeader().setDefaultSectionSize(80)
		self.verticalHeader().setVisible(False)
		
		self.setSelectionMode(QAbstractItemView.ExtendedSelection)
		self.setSelectionBehavior(QAbstractItemView.SelectRows)

		self.setDragEnabled(True)
		self.setDragDropMode(QAbstractItemView.DragDrop)
	
	def	sort(self,sort_index):
		self.selected_index=[]
		self.clearSelection()

		if self.sort_times==0:
			self.horizontalHeader().setSortIndicator(sort_index,Qt.AscendingOrder)
			self.sortItems(sort_index,Qt.AscendingOrder)
			self.sort_times+=1
		elif self.sort_times==1:
			self.horizontalHeader().setSortIndicator(sort_index,Qt.DescendingOrder)
			self.sortItems(sort_index,Qt.DescendingOrder)
			self.sort_times+=1
		elif self.sort_times==2:
			self.horizontalHeader().setSortIndicator(-1,Qt.AscendingOrder)
			self.sortItems(-1,Qt.AscendingOrder)
			self.sort_times=0
			self.sortReset.emit()
	
	def dropEvent(self, event:QDropEvent):
		
		if not event.isAccepted() and event.source() == self:
			drop_row = self.drop_on(event)

			rows = sorted(set(item.row() for item in self.selectedItems()))
			rows_to_move = [[QTableWidgetItem(self.item(row_index, column_index)) for column_index in range(self.columnCount())]
							for row_index in rows]
			for row_index in reversed(rows):
				self.removeRow(row_index)
				if row_index < drop_row:
					drop_row -= 1

			for row_index, data in enumerate(rows_to_move):
				row_index += drop_row
				self.insertRow(row_index)
				for column_index, column_data in enumerate(data):
					self.setItem(row_index, column_index, column_data)
			event.accept()

			for row_index in range(len(rows_to_move)):   # maybe can be done smarter
				for col in range(self.columnCount()):
					self.item(drop_row + row_index, col).setSelected(True)
		
	def drop_on(self, event):
		index = self.indexAt(event.pos())
		if not index.isValid():
			return self.rowCount()
		return index.row() + 1 if self.is_below(event.pos(), index) else index.row()

	def is_below(self, pos, index):
		rect = self.visualRect(index)
		margin = 2
		if pos.y() - rect.top() < margin:
			return False
		elif rect.bottom() - pos.y() < margin:
			return True
		# noinspection PyTypeChecker
		return rect.contains(pos, True) and not (int(self.model().flags(index)) & Qt.ItemIsDropEnabled) and pos.y() >= rect.center().y()

	def setColumn(self,column_list:list):
		self.setColumnCount(len(column_list))
		self.setHorizontalHeaderLabels(column_list)
	
	def StoreTableStatus(self):
		self.sort_index=self.horizontalHeader().sortIndicatorSection()
		self.sort_order=self.horizontalHeader().sortIndicatorOrder()
		
		self.selected_index=self.selectedIndexes()
		self.scrollbarValue=self.verticalScrollBar().value()
	
	def Clear(self):
		self.clearContents()
		self.setRowCount(0)
		self.horizontalHeader().setSortIndicator(-1,Qt.SortOrder.AscendingOrder) #因为如果有排序的话，再insertItem会有疏漏，所以提前取消排序

	def RestoreTableStatus(self):
		# restore sorting
		
		self.horizontalHeader().setSortIndicator(self.sort_index,self.sort_order)
		self.sortItems(self.sort_index,self.sort_order)
		
		# restore selection
		for index in self.selected_index:
			self.selectionModel().select(index,QItemSelectionModel.Select | QItemSelectionModel.Rows)
		
		# restore scrollbar
		self.verticalScrollBar().setValue(self.scrollbarValue)

	def addRow(self, row:int, column_item_list:list):
		
		if len(column_item_list)>self.columnCount():
			raise("column_item_list out of bounds")
		
		self.insertRow(row)
		i=0
		for item in column_item_list:
			self.setItem(row,i,item)
			i+=1
	
	def findRowInColumn(self,):
		pass


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'Ui_DemoCentralWidget2.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

import DTPySide.DT_rc

class Ui_DemoCentralWidget2(object):
    def setupUi(self, DemoCentralWidget2):
        if not DemoCentralWidget2.objectName():
            DemoCentralWidget2.setObjectName(u"DemoCentralWidget2")
        DemoCentralWidget2.resize(400, 300)
        self.actionHello_World = QAction(DemoCentralWidget2)
        self.actionHello_World.setObjectName(u"actionHello_World")
        icon = QIcon()
        icon.addFile(u":/icons/icon/holoicon2.ico", QSize(), QIcon.Normal, QIcon.Off)
        self.actionHello_World.setIcon(icon)
        self.horizontalLayout = QHBoxLayout(DemoCentralWidget2)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.homelabel = QLabel(DemoCentralWidget2)
        self.homelabel.setObjectName(u"homelabel")
        self.homelabel.setAlignment(Qt.AlignCenter)

        self.horizontalLayout.addWidget(self.homelabel)


        self.retranslateUi(DemoCentralWidget2)

        QMetaObject.connectSlotsByName(DemoCentralWidget2)
    # setupUi

    def retranslateUi(self, DemoCentralWidget2):
        DemoCentralWidget2.setWindowTitle(QCoreApplication.translate("DemoCentralWidget2", u"DemoCentralWidget2", None))
        self.actionHello_World.setText(QCoreApplication.translate("DemoCentralWidget2", u"Hello World", None))
        self.homelabel.setText("")
    # retranslateUi




from PySide2.QtWidgets import *
from PySide2.QtGui import *
from PySide2.QtCore import *
import sys, os, shutil
import re
import copy
import time
import requests
import urllib #淦！最新版的urllib3不支持代理。pip install urllib3==1.25.11就行了
from lxml import etree
import pickle
import json
import pypinyin
from functools import partial
from win32.lib import win32con
from win32 import win32gui, win32api
from win32com.shell import shell,shellcon
import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import colour

def ShowUp(widget:QWidget):
	if widget.isFullScreen():
		widget.showFullScreen()
		widget.TitleBar.hide()
	else:
		widget.showNormal()
		widget.TitleBar.show()
	
	widget.activateWindow()

def MoveToCenterOfScreen(widget:QWidget):
	"""在initializeWindow中，setCentralWidget之后调用
	"""
	widget.move((win32api.GetSystemMetrics(0)-widget.width())//2,(win32api.GetSystemMetrics(1)-widget.height())//2)

def IconFromCurrentTheme(name):
	return QIcon(":/icons/%s/%s"%(QIcon.themeName(),name))

def Clear_Layout(layout):
	for i in reversed(range(layout.count())):
		item=layout.itemAt(i)
		
		if item.spacerItem():
			layout.removeItem(item)
			del item
		else:
			item.widget().deleteLater()

def Font_Resize(font,resize_ratio):
	new_font=QFont(font)
	fontsize=font.pointSize()
	fontsize=int(fontsize*resize_ratio)
	new_font.setPointSize(fontsize)
	return new_font

def Delay_Msecs(msecs):
	"传入int类型的延迟毫秒数msecs"
	dieTime= QTime.currentTime().addMSecs(msecs)
	while QTime.currentTime() < dieTime:
		QCoreApplication.processEvents(QEventLoop.AllEvents, 100)

def QDate_to_Tuple(date:QDate):
	return (date.year(),date.month(),date.day())

def WhatDayIsToday(mode):
	"""返回年月日

	Args:
		mode (int, optional):选择返回的类型。

		0：Tuple

		1: QDate
	"""
	today=time.localtime()
	if mode==0:
		return today.tm_year,today.tm_mon,today.tm_mday
	elif mode==1:
		return QDate(today.tm_year,today.tm_mon,today.tm_mday)

def Generate_ConicalGradientColor(colorList,cube_width):
	n=len(colorList)
	if n==0:
		return QColor("#5B1803")
	
	angle=0
	delta=1/n
	colors=[]
	for i in range(n):
		colors.append((angle,colorList[i]))
		colors.append((angle+delta-0.01,colorList[i]))
		angle+=delta

	color=QConicalGradient(cube_width/2,cube_width/2,90)
	color.setStops(colors)
	return color

def show_ContextMenu_Beneath(menu,btn):
	btn_pos=btn.pos()
	btn_height=btn.height()
	btn_pos+=QPoint(0,btn_height)
	true_pos=btn.parentWidget().mapToGlobal(btn_pos)
	menu.exec_(true_pos)

def show_ContextMenu_Right(menu,btn):
	btn_pos=btn.pos()
	btn_height=btn.height()
	btn_pos+=QPoint(btn_height,0)
	true_pos=btn.parentWidget().mapToGlobal(btn_pos)
	menu.exec_(true_pos)

def Generate_Key(password):
	"""
	根据password生成一个固定的salt，用salt生成一个PBKDF2，用PBKDF2和password生成key
	所以给定一个固定的password，将返回那个固定的key。
	"""
	salt=password.encode()[::-1]
	password=password.encode()
	kdf=PBKDF2HMAC(algorithm=hashes.SHA256(),length=32,salt=salt,iterations=100000,backend=default_backend())
	key=base64.urlsafe_b64encode(kdf.derive(password))
	return key

def Fernet_Encrypt_Save(password,data,file_path):
	try:
		data=pickle.dumps(data)
		key=Generate_Key(password)

		fer=Fernet(key)
		encrypt_data=fer.encrypt(data)

		if not os.path.exists(os.path.dirname(file_path)):
			os.makedirs(os.path.dirname(file_path))
		
		with open(file_path,"wb") as f:
			f.write(encrypt_data)
		
		return True
	except:
		return False

def Fernet_Decrypt_Load(password,file_path):
	try:
		key=Generate_Key(password)
	
		with open(file_path,"rb") as f:
			data=f.read()
		
		fer=Fernet(key)
		decrypt_data=fer.decrypt(data)
		decrypt_data=pickle.loads(decrypt_data)

		return decrypt_data
	except:
		return False

def Fernet_Encrypt(password,data):
	try:
		data=pickle.dumps(data)
		key=Generate_Key(password)

		fer=Fernet(key)
		encrypt_data=fer.encrypt(data)
		
		return encrypt_data
	except:
		return False

def Fernet_Decrypt(password,data):
	try:
		key=Generate_Key(password)
		
		fer=Fernet(key)
		decrypt_data=fer.decrypt(data)
		decrypt_data=pickle.loads(decrypt_data)

		return decrypt_data
	except:
		return False

def Json_Save(data,file_path):
	try:
		with open(file_path,"w",encoding="utf-8") as f:
			json.dump(data,f,ensure_ascii=False,indent=4)
		
		return True
	except:
		return False

def Json_Load(file_path):
	try:
		with open(file_path,"r",encoding="utf-8") as f:
			data=json.load(f)
		return data
	except:
		return False

def Str_to_AZ(input):
	
	def cn_to_az(last_name):
		rows = pypinyin.pinyin(last_name, style=pypinyin.NORMAL)
		return ''.join(row[0][0] for row in rows if len(row) > 0)

	def jp_to_az(i):
		jp1=["”","“","《","》","あ","い","う","え","お","か","き","く","け","こ","さ","し","す","せ","そ","た","ち","つ","て","と","な","に","ぬ","ね","の","は","ひ","ふ","へ","ほ","ま","み","む","め","も","や","ゆ","よ","ら","り","る","れ","ろ","わ","を","ん","が","ぎ","ぐ","げ","ご","ざ","じ","ず","ぜ","ぞ","だ","ぢ","づ","で","ど","ば","び","ぶ","べ","ぼ","ぱ","ぴ","ぷ","ぺ","ぽ"]
		jp2=["”","“","《","》","ア","イ","ウ","エ","オ","カ","キ","ク","ケ","コ","サ","シ","ス","セ","ソ","タ","チ","ツ","テ","ト","ナ","ニ","ヌ","ネ","ノ","ハ","ヒ","フ","ヘ","ホ","マ","ミ","ム","メ","モ","ヤ","ユ","ヨ","ラ","リ","ル","レ","ロ","ワ","ヲ","ン","ガ","ギ","グ","ゲ","ゴ","ザ","ジ","ズ","ゼ","ゾ","ダ","ヂ","ヅ","デ","ド","バ","ビ","ブ","ベ","ボ","パ","ピ","プ","ペ","ポ"]
		az=["”","“","《","》","a","i","u","e","o","ka","ki","ku","ke","ko","sa","si","su","se","so","ta","ti","tu","te","to","na","ni","nu","ne","no","ha","hi","hu","he","ho","ma","mi","mu","me","mo","ya","yu","yo","ra","ri","ru","re","ro","wa","wo","n","ga","gi","gu","ge","go","za","ji","zu","ze","zo","da","di","du","de","do","ba","bi","bu","be","bo","pa","pi","pu","pe","po"]
		try:
			n=jp1.index(i)
			return az[n]
		except:
			try:
				n=jp2.index(i)
				return az[n]
			except:
				pass
				# print("%s	假名好像不完整\n"%i)
		return ""
	
	output=""
	
	#用unicode划分语言区
	for i in input:
		if re.match(r"[\u0000-\u007F]",i):#英
			output+=i.lower()
		elif re.match(r"[\u4E00-\u9FFF]",i):#中
			output+=cn_to_az(i[0])
		elif re.match(r"[\u0800-\u4DFF]",i):#日，\u4E00是中文的一
			output+=jp_to_az(i)
		# if re.match(r"[\uAC00-\uD7FF]",c)#韩
		else:
			output+=i
		
	return output

def List_Intersection(a:list, b:list) -> list:
	return list(set(a).intersection(set(b)))

def List_Union(a:list, b:list) -> list:
	return list(set(a).union(set(b)))

def List_Difference(a:list, b:list) -> list:
	return list(set(a).difference(set(b)))
	
def List_Symmetric_Difference(a:list, b:list) -> list:
	return list(set(a).symmetric_difference(set(b)))

def List_Intersection_Full(a:list, b:list) -> list:
	c=[]
	for i in a:
		if i in b:
			c.append(i)
	return c

def List_Union_Full(a:list, b:list) -> list:
	c=copy.deepcopy(a)
	for i in b:
		if i not in c:
			c.append(i)
	return c

def List_Difference_Full(a:list, b:list) -> list:
	c=[]
	for i in a:
		if i not in b:
			c.append(i)
	return c

def List_Symmetric_Difference_Full(a:list, b:list) -> list:
	c=[]
	for i in a:
		if i not in b:
			c.append(i)
	for i in b:
		if i not in a:
			c.append(i)
	return c

def GetWebPageResponse(url,cookie="") -> requests.Response:
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	if response.ok:
		return response
	else:
		return None

def GetWebPageHTML(url,cookie=""):
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	if response.ok:
		return response.text
	else:
		return None

def GetWebPageHeaders(url,cookie=""):
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	if response.ok:
		return response.headers
	else:
		return None

def GetWebPageType(url,cookie=""):
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	if response.ok:
		return response["Content-Type"]
	else:
		return None

def GetWebPageTitle(url,cookie=""):
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	if response.ok:
		html=response.text
		try:
			tree=etree.HTML(html)
			title=tree.xpath(".//title/text()")[0]
			title=urllib.parse.unquote(title,'utf-8')
			return str(title)
		except:
			return None
	else:
		return None


def GetWebFavIcon(url,cookie=""):
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	if response.ok:
		try:
			html=response.text
			tree=etree.HTML(html)
			icon_url=tree.xpath("//*[@rel='icon']/@href | //*[@rel='shortcut icon']/@href")[-1]
			if "http" not in icon_url:
				icon_url=urllib.parse.urljoin(url,icon_url)
			res=GetWebPageResponse(icon_url)
			if res!=None:
				return res.content
			else:
				return None
		except:
			return None
	else:
		return None

def GetWebPagePic(url,cookie=""):
	head={}
	head["cookie"]=cookie
	head['User-Agent']='Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.135 YaBrowser/21.6.2.143 (beta) Yowser/2.5 Safari/537.36'
	try:
		response=requests.get(url,headers=head,timeout=3)
	except:
		return None
	
	if response.ok:
		return response.content
	else:
		return None

def Delete_to_Recyclebin(dir):
	"删除成功返回True"
	result = shell.SHFileOperation((0,shellcon.FO_DELETE,dir,None, shellcon.FOF_SILENT | shellcon.FOF_ALLOWUNDO | shellcon.FOF_NOCONFIRMATION,None,None))  #删除文件到回收站
	return result[0]==0

def Win32_Shellcopy(src, dest):
	"""
	Copy files and directories using Windows shell.

	:param src: Path or a list of paths to copy. Filename portion of a path
				(but not directory portion) can contain wildcards ``*`` and
				``?``.
	:param dst: destination directory.
	:returns: ``True`` if the operation completed successfully,
			  ``False`` if it was aborted by user (completed partially).
	:raises: ``WindowsError`` if anything went wrong. Typically, when source
			 file was not found.

	.. seealso:
		`SHFileperation on MSDN <http://msdn.microsoft.com/en-us/library/windows/desktop/bb762164(v=vs.85).aspx>`
	"""
	if isinstance(src, str):
		src = os.path.abspath(src)
	else:  # iterable
		src = '\0'.join(os.path.abspath(path) for path in src)

	result, aborted = shell.SHFileOperation((
		0,
		shellcon.FO_COPY,
		src,
		os.path.abspath(dest),
		shellcon.FOF_NOCONFIRMMKDIR,  # flags
		None,
		None))

	if not aborted and result != 0:
		# Note: raising a WindowsError with correct error code is quite
		# difficult due to SHFileOperation historical idiosyncrasies.
		# Therefore we simply pass a message.
		raise WindowsError('SHFileOperation failed: 0x%08x' % result)

	return not aborted

def Win32_Shellmove(src, dest):
	"""
	Move files and directories using Windows shell.

	:param src: Path or a list of paths to move. Filename portion of a path
				(but not directory portion) can contain wildcards ``*`` and
				``?``.
	:param dst: destination directory.
	:returns: ``True`` if the operation completed successfully,
			  ``False`` if it was aborted by user (completed partially).
	:raises: ``WindowsError`` if anything went wrong. Typically, when source
			 file was not found.

	.. seealso:
		`SHFileperation on MSDN <http://msdn.microsoft.com/en-us/library/windows/desktop/bb762164(v=vs.85).aspx>`
	"""
	if isinstance(src, str):
		src = os.path.abspath(src)
	else:  # iterable
		src = '\0'.join(os.path.abspath(path) for path in src)

	result, aborted = shell.SHFileOperation((
		0,
		shellcon.FO_MOVE,
		src,
		os.path.abspath(dest),
		shellcon.FOF_NOCONFIRMMKDIR,  # flags
		None,
		None))

	if not aborted and result != 0:
		# Note: raising a WindowsError with correct error code is quite
		# difficult due to SHFileOperation historical idiosyncrasies.
		# Therefore we simply pass a message.
		raise WindowsError('SHFileOperation failed: 0x%08x' % result)

	return not aborted



#############################################################################
##
## Copyright (C) 2020 The Qt Company Ltd.
## Contact: http://www.qt.io/licensing/
##
## This file is part of the Qt for Python examples of the Qt Toolkit.
##
## $QT_BEGIN_LICENSE:BSD$
## You may use this file under the terms of the BSD license as follows:
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are
## met:
##   * Redistributions of source code must retain the above copyright
##     notice, this list of conditions and the following disclaimer.
##   * Redistributions in binary form must reproduce the above copyright
##     notice, this list of conditions and the following disclaimer in
##     the documentation and/or other materials provided with the
##     distribution.
##   * Neither the name of The Qt Company Ltd nor the names of its
##     contributors may be used to endorse or promote products derived
##     from this software without specific prior written permission.
##
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
## $QT_END_LICENSE$
##
#############################################################################

from DTPySide import *


POEM = """Twinkle, twinkle, little star,
How I wonder what you are.
Up above the world so high,
Like a diamond in the sky.
Twinkle, twinkle, little star,
How I wonder what you arenot"""

DIR_OPEN_ICON = ":/qt-project.org/styles/commonstyle/images/diropen-128.png"

COMPUTER_ICON = ":/qt-project.org/styles/commonstyle/images/computer-32.png"

SYSTEMINFO = """<html><head/><body>
<h3>Python</h3><p>{}</p>
<h3>Qt Build</h3><p>{}</p>
<h3>Operating System</h3><p>{}</p>
<h3>Screens</h3>
{}
</body></html>"""


def class_name(o):
    return o.metaObject().className()


def help_url(page):
    """Build a Qt help URL from the page name"""
    major_version = qVersion().split('.')[0]
    return "https://doc.qt.io/qt-{}/{}.html".format(major_version, page)


def launch_help(widget):
    """Launch a widget's help page"""
    url = help_url(class_name(widget).lower())
    QDesktopServices.openUrl(url)


def launch_module_help():
    QDesktopServices.openUrl(help_url("qtwidgets-index"))


def init_widget(w, name):
    """Init a widget for the gallery, give it a tooltip showing the
       class name"""
    w.setObjectName(name)
    w.setToolTip(class_name(w))


def style_names():
    """Return a list of styles, default platform style first"""
    default_style_name = QApplication.style().objectName().lower()
    result = []
    for style in QStyleFactory.keys():
        if style.lower() == default_style_name:
            result.insert(0, style)
        else:
            result.append(style)
    return result


def embed_into_hbox_layout(w, margin=5):
    """Embed a widget into a layout to give it a frame"""
    result = QWidget()
    layout = QHBoxLayout(result)
    layout.setContentsMargins(margin, margin, margin, margin)
    layout.addWidget(w)
    return result


def format_geometry(rect):
    """Format a geometry as a X11 geometry specification"""
    return "{}x{}{:+d}{:+d}".format(rect.width(), rect.height(),
                                    rect.x(), rect.y())


def screen_info(widget):
    """Format information on the screens"""
    policy = QGuiApplication.highDpiScaleFactorRoundingPolicy()
    policy_string = str(policy).split('.')[-1]
    result = "<p>High DPI scale factor rounding policy: {}</p><ol>".format(policy_string)
    for screen in QGuiApplication.screens():
        current = screen == widget.screen()
        result += "<li>"
        if current:
            result += "<i>"
        result += '"{}" {} {}DPI, DPR={}'.format(screen.name(),
                                                 format_geometry(screen.geometry()),
                                                 int(screen.logicalDotsPerInchX()),
                                                 screen.devicePixelRatio())
        if current:
            result += "</i>"
        result += "</li>"
    result += "</ol>"
    return result


class WidgetGallery(QWidget):
    """Dialog displaying a gallery of Qt Widgets"""

    def __init__(self,parent):
        super(WidgetGallery, self).__init__(parent=parent)

        self._progress_bar = self.create_progress_bar()

        self._style_combobox = QComboBox()
        init_widget(self._style_combobox, "styleComboBox")
        self._style_combobox.addItems(style_names())

        style_label = QLabel("Style:")
        init_widget(style_label, "style_label")
        style_label.setBuddy(self._style_combobox)

        help_label = QLabel("Press F2 over a widget to see Documentation")
        init_widget(help_label, "help_label")

        disable_widgets_checkbox = QCheckBox("Disable widgets")
        init_widget(disable_widgets_checkbox, "disable_widgets_checkbox")

        buttons_groupbox = self.create_buttons_groupbox()
        itemview_tabwidget = self.create_itemview_tabwidget()
        simple_input_widgets_groupbox = self.create_simple_inputwidgets_groupbox()
        text_toolbox = self.create_text_toolbox()

        self._style_combobox.textActivated.connect(self.change_style)
        disable_widgets_checkbox.toggled.connect(buttons_groupbox.setDisabled)
        disable_widgets_checkbox.toggled.connect(text_toolbox.setDisabled)
        disable_widgets_checkbox.toggled.connect(itemview_tabwidget.setDisabled)
        disable_widgets_checkbox.toggled.connect(simple_input_widgets_groupbox.setDisabled)

        help_shortcut = QShortcut(self)
        help_shortcut.setKey(QKeySequence(Qt.Key_F2))
        help_shortcut.activated.connect(self.help_on_current_widget)

        top_layout = QHBoxLayout()
        top_layout.addWidget(style_label)
        top_layout.addWidget(self._style_combobox)
        top_layout.addStretch(1)
        top_layout.addWidget(help_label)
        top_layout.addStretch(1)
        top_layout.addWidget(disable_widgets_checkbox)

        self.main_layout = QGridLayout(self)
        self.main_layout.addLayout(top_layout, 0, 0, 1, 2)
        self.main_layout.addWidget(buttons_groupbox, 1, 0)
        self.main_layout.addWidget(simple_input_widgets_groupbox, 1, 1)
        self.main_layout.addWidget(itemview_tabwidget, 2, 0)
        self.main_layout.addWidget(text_toolbox, 2, 1)
        self.main_layout.addWidget(self._progress_bar, 3, 0, 1, 2)

        self.setWindowTitle("Widget Gallery Qt {}".format(qVersion()))

    def change_style(self, style_name):
        QApplication.setStyle(QStyleFactory.create(style_name))

    def advance_progressbar(self):
        cur_val = self._progress_bar.value()
        max_val = self._progress_bar.maximum()
        self._progress_bar.setValue(cur_val + (max_val - cur_val) / 100)

    def create_buttons_groupbox(self):
        result = QGroupBox("Buttons")
        init_widget(result, "buttons_groupbox")

        default_pushbutton = QPushButton("Default Push Button")
        init_widget(default_pushbutton, "default_pushbutton")
        default_pushbutton.setDefault(True)

        toggle_pushbutton = QPushButton("Toggle Push Button")
        init_widget(toggle_pushbutton, "toggle_pushbutton")
        toggle_pushbutton.setCheckable(True)
        toggle_pushbutton.setChecked(True)

        flat_pushbutton = QPushButton("Flat Push Button")
        init_widget(flat_pushbutton, "flat_pushbutton")
        flat_pushbutton.setFlat(True)

        toolbutton = QToolButton()
        init_widget(toolbutton, "toolButton")
        toolbutton.setText("Tool Button")

        menu_toolbutton = QToolButton()
        init_widget(menu_toolbutton, "menuButton")
        menu_toolbutton.setText("Menu Button")
        tool_menu = QMenu(menu_toolbutton)
        menu_toolbutton.setPopupMode(QToolButton.InstantPopup)
        tool_menu.addAction("Option")
        tool_menu.addSeparator()
        action = tool_menu.addAction("Checkable Option")
        action.setCheckable(True)
        menu_toolbutton.setMenu(tool_menu)
        tool_layout = QHBoxLayout()
        tool_layout.addWidget(toolbutton)
        tool_layout.addWidget(menu_toolbutton)

        commandlinkbutton = QCommandLinkButton("Command Link Button")
        init_widget(commandlinkbutton, "commandLinkButton")
        commandlinkbutton.setDescription("Description")

        button_layout = QVBoxLayout()
        button_layout.addWidget(default_pushbutton)
        button_layout.addWidget(toggle_pushbutton)
        button_layout.addWidget(flat_pushbutton)
        button_layout.addLayout(tool_layout)
        button_layout.addWidget(commandlinkbutton)
        button_layout.addStretch(1)

        radiobutton_1 = QRadioButton("Radio button 1")
        init_widget(radiobutton_1, "radioButton1")
        radiobutton_2 = QRadioButton("Radio button 2")
        init_widget(radiobutton_2, "radioButton2")
        radiobutton_3 = QRadioButton("Radio button 3")
        init_widget(radiobutton_3, "radioButton3")
        radiobutton_1.setChecked(True)

        checkbox = QCheckBox("Tri-state check box")
        init_widget(checkbox, "checkBox")
        checkbox.setTristate(True)
        checkbox.setCheckState(Qt.PartiallyChecked)

        checkableLayout = QVBoxLayout()
        checkableLayout.addWidget(radiobutton_1)
        checkableLayout.addWidget(radiobutton_2)
        checkableLayout.addWidget(radiobutton_3)
        checkableLayout.addWidget(checkbox)
        checkableLayout.addStretch(1)

        self.main_layout = QHBoxLayout(result)
        self.main_layout.addLayout(button_layout)
        self.main_layout.addLayout(checkableLayout)
        self.main_layout.addStretch()
        return result

    def create_text_toolbox(self):
        result = QToolBox()
        init_widget(result, "toolBox")

        # Create centered/italic HTML rich text
        rich_text = "<html><head/><body><i>"
        for line in POEM.split('\n'):
            rich_text += "<center>" + line + "</center>"
        rich_text += "</i></body></html>"

        text_edit = QTextEdit(rich_text)
        init_widget(text_edit, "textEdit")
        plain_textedit = QPlainTextEdit(POEM)
        init_widget(plain_textedit, "plainTextEdit")

        self._systeminfo_textbrowser = QTextBrowser()
        init_widget(self._systeminfo_textbrowser, "systemInfoTextBrowser")
        self.update_systeminfo()

        result.addItem(embed_into_hbox_layout(text_edit), "Text Edit")
        result.addItem(embed_into_hbox_layout(plain_textedit),
                       "Plain Text Edit")
        result.addItem(embed_into_hbox_layout(self._systeminfo_textbrowser),
                       "Text Browser")
        return result

    def create_itemview_tabwidget(self):
        result = QTabWidget()
        init_widget(result, "bottomLeftTabWidget")
        result.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Ignored)

        tree_view = QTreeView()
        tree_view.setSortingEnabled(True)
        init_widget(tree_view, "treeView")
        filesystem_model = QFileSystemModel(tree_view)
        filesystem_model.setRootPath(QDir.rootPath())
        tree_view.setModel(filesystem_model)

        table_widget = QTableWidget()
        init_widget(table_widget, "tableWidget")
        table_widget.setRowCount(10)
        table_widget.setColumnCount(10)

        list_model = QStandardItemModel(0, 0, result)

        list_model.appendRow([QStandardItem(QIcon(COMPUTER_ICON), "Computer"),QStandardItem(QIcon(DIR_OPEN_ICON), "Directory")])
        list_model.appendRow([QStandardItem(QIcon(DIR_OPEN_ICON), "Directory"),QStandardItem(QIcon(COMPUTER_ICON), "Computer")])
        list_model.appendRow([QStandardItem(DTIcon.HoloIcon1(), "Holo"),QStandardItem(DTIcon.HoloIcon2(), "Holo"),QStandardItem(DTIcon.Heart(), "Heart")])

        list_view = QListView()
        init_widget(list_view, "listView")
        list_view.setModel(list_model)

        icon_mode_listview = QListView()
        init_widget(icon_mode_listview, "iconModeListView")

        icon_mode_listview.setViewMode(QListView.IconMode)
        icon_mode_listview.setModel(list_model)

        table_view=QTableView()
        init_widget(table_view, "tableview")
        table_view.setModel(list_model)

        result.addTab(embed_into_hbox_layout(tree_view), "Tree View")
        result.addTab(embed_into_hbox_layout(table_widget), "Table")
        result.addTab(embed_into_hbox_layout(list_view), "List View")
        result.addTab(embed_into_hbox_layout(icon_mode_listview),
                      "List View")
        result.addTab(embed_into_hbox_layout(table_view),
                      "Tabel View")


        return result

    def create_simple_inputwidgets_groupbox(self):
        result = QGroupBox("Simple Input Widgets")
        init_widget(result, "bottomRightGroupBox")
        result.setCheckable(True)
        result.setChecked(True)

        lineedit = QLineEdit("s3cRe7")
        init_widget(lineedit, "lineEdit")
        lineedit.setClearButtonEnabled(True)
        lineedit.setEchoMode(QLineEdit.Password)

        spin_box = QSpinBox()
        init_widget(spin_box, "spinBox")
        spin_box.setValue(50)

        date_timeedit = QDateTimeEdit()
        init_widget(date_timeedit, "dateTimeEdit")
        date_timeedit.setDateTime(QDateTime.currentDateTime())

        slider = QSlider()
        init_widget(slider, "slider")
        slider.setOrientation(Qt.Horizontal)
        slider.setValue(40)

        scrollbar = QScrollBar()
        init_widget(scrollbar, "scrollBar")
        scrollbar.setOrientation(Qt.Horizontal)
        scrollbar.setValue(60)

        dial = QDial()
        init_widget(dial, "dial")
        dial.setValue(30)
        dial.setNotchesVisible(True)

        layout = QGridLayout(result)
        layout.addWidget(lineedit, 0, 0, 1, 2)
        layout.addWidget(spin_box, 1, 0, 1, 2)
        layout.addWidget(date_timeedit, 2, 0, 1, 2)
        layout.addWidget(slider, 3, 0)
        layout.addWidget(scrollbar, 4, 0)
        layout.addWidget(dial, 3, 1, 2, 1)
        layout.setRowStretch(5, 1)
        return result

    def create_progress_bar(self):
        result = QProgressBar()
        init_widget(result, "progressBar")
        result.setRange(0, 10000)
        result.setValue(0)

        timer = QTimer(self)
        timer.timeout.connect(self.advance_progressbar)
        timer.start(1000)
        return result

    def update_systeminfo(self):
        """Display system information"""
        system_info = SYSTEMINFO.format(sys.version,
                                        QLibraryInfo.build(),
                                        QSysInfo.prettyProductName(),
                                        screen_info(self))
        self._systeminfo_textbrowser.setHtml(system_info)

    def help_on_current_widget(self):
        """Display help on widget under mouse"""
        w = QApplication.widgetAt(QCursor.pos(self.screen()))
        while w:  # Skip over internal widgets
            name = w.objectName()
            if name and not name.startswith("qt_"):
                launch_help(w)
                break
            w = w.parentWidget()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'Ui_DTTitleBar.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

import DTPySide.DT_rc

class Ui_DTTitleBar(object):
    def setupUi(self, DTTitleBar):
        if not DTTitleBar.objectName():
            DTTitleBar.setObjectName(u"DTTitleBar")
        DTTitleBar.resize(356, 42)
        DTTitleBar.setStyleSheet(u"")
        self.horizontalLayout = QHBoxLayout(DTTitleBar)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.TitleBarFrame = QFrame(DTTitleBar)
        self.TitleBarFrame.setObjectName(u"TitleBarFrame")
        self.TitleBarFrame.setStyleSheet(u"")
        self.horizontalLayout_2 = QHBoxLayout(self.TitleBarFrame)
        self.horizontalLayout_2.setSpacing(4)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(9, 3, 9, 3)
        self.title_icon = QPushButton(self.TitleBarFrame)
        self.title_icon.setObjectName(u"title_icon")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.title_icon.sizePolicy().hasHeightForWidth())
        self.title_icon.setSizePolicy(sizePolicy)
        self.title_icon.setMinimumSize(QSize(36, 36))
        self.title_icon.setMaximumSize(QSize(36, 36))
        self.title_icon.setStyleSheet(u"")

        self.horizontalLayout_2.addWidget(self.title_icon)

        self.label_titlebar = QLabel(self.TitleBarFrame)
        self.label_titlebar.setObjectName(u"label_titlebar")
        sizePolicy1 = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.label_titlebar.sizePolicy().hasHeightForWidth())
        self.label_titlebar.setSizePolicy(sizePolicy1)
        font = QFont()
        font.setFamily(u"Segoe UI")
        font.setPointSize(18)
        font.setBold(True)
        font.setWeight(75)
        self.label_titlebar.setFont(font)

        self.horizontalLayout_2.addWidget(self.label_titlebar)

        self.btn_minimize = QPushButton(self.TitleBarFrame)
        self.btn_minimize.setObjectName(u"btn_minimize")
        sizePolicy.setHeightForWidth(self.btn_minimize.sizePolicy().hasHeightForWidth())
        self.btn_minimize.setSizePolicy(sizePolicy)
        self.btn_minimize.setMinimumSize(QSize(24, 24))
        self.btn_minimize.setMaximumSize(QSize(24, 24))
        self.btn_minimize.setStyleSheet(u"")
        self.btn_minimize.setIconSize(QSize(24, 24))

        self.horizontalLayout_2.addWidget(self.btn_minimize)

        self.btn_maximize = QPushButton(self.TitleBarFrame)
        self.btn_maximize.setObjectName(u"btn_maximize")
        sizePolicy.setHeightForWidth(self.btn_maximize.sizePolicy().hasHeightForWidth())
        self.btn_maximize.setSizePolicy(sizePolicy)
        self.btn_maximize.setMinimumSize(QSize(24, 24))
        self.btn_maximize.setMaximumSize(QSize(24, 24))
        self.btn_maximize.setStyleSheet(u"")
        self.btn_maximize.setIconSize(QSize(24, 24))

        self.horizontalLayout_2.addWidget(self.btn_maximize)

        self.btn_close = QPushButton(self.TitleBarFrame)
        self.btn_close.setObjectName(u"btn_close")
        sizePolicy.setHeightForWidth(self.btn_close.sizePolicy().hasHeightForWidth())
        self.btn_close.setSizePolicy(sizePolicy)
        self.btn_close.setMinimumSize(QSize(24, 24))
        self.btn_close.setMaximumSize(QSize(24, 24))
        self.btn_close.setStyleSheet(u"")
        self.btn_close.setIconSize(QSize(24, 24))

        self.horizontalLayout_2.addWidget(self.btn_close)


        self.horizontalLayout.addWidget(self.TitleBarFrame)


        self.retranslateUi(DTTitleBar)

        QMetaObject.connectSlotsByName(DTTitleBar)
    # setupUi

    def retranslateUi(self, DTTitleBar):
        DTTitleBar.setWindowTitle(QCoreApplication.translate("DTTitleBar", u"DTTitleBar", None))
        self.label_titlebar.setText(QCoreApplication.translate("DTTitleBar", u"DT", None))
        self.btn_minimize.setText("")
        self.btn_maximize.setText("")
        self.btn_close.setText("")
    # retranslateUi




from DTPySide import *

from DTPySide.demo.demoSession import DemoMainSession

def run():

	app=DTAPP(sys.argv)
	
	app.setAuthor("Holence")
	app.setApplicationName("DTPySide Demo")
	app.setLoginEnable(True)
	app.loadTranslation()
	mainsession=DemoMainSession(app)
	app.setMainSession(mainsession)
	# app.debugRun("123",True)
	app.run()

if __name__=="__main__":
	run()


from __future__ import annotations
from DTPySide import *

# MessageBox Module
from DTPySide.DTFrame.Ui_DTMessageBox import Ui_DTMessageBox
from DTPySide.DTFrame.DTDialog import DTDialog

class MessageModule(Ui_DTMessageBox,QWidget):
	def __init__(self,parent):
		super().__init__(parent)
		self.setupUi(self)

class DTMessageBox(DTDialog):
	def __init__(self,parent,title,text,icon=None,detail=""):
		super().__init__(parent,title)
		self.module=MessageModule(self)
		self.setCentralWidget(self.module)

		self.module.label_text.setText(text)
		# self.module.label_text.setOpenExternalLinks(True)

		if detail=="":
			self.module.textBrowser.hide()
		else:
			self.module.textBrowser.setText(detail)
			self.module.textBrowser.setStyleSheet('font-family:"微软雅黑"; font-size:12pt;')

		self.setButtonBox(QDialogButtonBox.Ok)
		self.setDefaultButton(QDialogButtonBox.Ok)

		if icon!=None:
			icon_pic=icon.pixmap(QSize(64,64))
			self.module.label_icon.setPixmap(icon_pic)
		else:
			self.module.label_icon.hide()
			self.module.horizontalLayout.setContentsMargins(30,10,0,5)

		self.adjustSize()
		MoveToCenterOfScreen(self)

		self.exec_()

class DTConfirmBox(DTDialog):
	def __init__(self,parent,title,text,icon=None,detail=""):
		super().__init__(parent, title)
		self.module=MessageModule(self)
		self.setCentralWidget(self.module)

		self.module.label_text.setText(text)

		if detail=="":
			self.module.textBrowser.hide()
		else:
			self.module.textBrowser.setText(detail)
			self.module.textBrowser.setStyleSheet('font-family:"微软雅黑"; font-size:12pt;')

		if icon!=None:
			icon_pic=icon.pixmap(QSize(64,64))
			self.module.label_icon.setPixmap(icon_pic)
		else:
			self.module.label_icon.setVisible(False)
			self.module.horizontalLayout.setContentsMargins(45,10,0,5)

		self.adjustSize()
		MoveToCenterOfScreen(self)


from __future__ import annotations
from DTPySide import *

# Login 
from DTPySide.DTModule.Ui_DTLogin import Ui_DTLogin
class DTLogin(Ui_DTLogin,QWidget):
	def __init__(self,parent):
		super().__init__(parent)
		self.setupUi(self)
		self.label_lock.setPixmap(IconFromCurrentTheme("lock.svg").pixmap(24,24))
		


import setuptools
import os

BUILDING_DIR=r"C:\D\DTPySide"

long_description=""
with open(os.path.join(BUILDING_DIR, 'README.md'), "r", encoding="utf-8") as fh:
    long_description += fh.read()
with open(os.path.join(BUILDING_DIR, 'CHANGELOG.md'), "r", encoding="utf-8") as fh:
    long_description += fh.read()

with open(os.path.join(BUILDING_DIR, 'CHANGELOG.md'), "r", encoding="utf-8") as fh:
    changelog = fh.read()
version=changelog.splitlines()[2][3:]

setuptools.setup(
    name="DTPySide",
    version=version,
    author="Holence",
    author_email="Holence08@gmail.com",
    description="A PySide framework for constructing customizable GUI software.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Holence/DongliTeahousePySide",
    project_urls={
        "Bug Tracker": "https://github.com/Holence/DongliTeahousePySide/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    include_package_data = True,
    python_requires=">=3.8",
    install_requires=[
        "colour",
        "cryptography",
        "lxml",
        "pypinyin",
        "PySide2",
        "pywin32",
        "requests",
        "urllib3"
    ]
)